##-----------------------------LICENSE NOTICE------------------------------------
##  This file is part of CPCtelera: An Amstrad CPC Game Engine
##  Copyright (C) 2016 ronaldo / Fremos / Cheesetea / ByteRealms (@FranGallegoBR)
##
##  This program is free software: you can redistribute it and/or modify
##  it under the terms of the GNU Lesser General Public License as published by
##  the Free Software Foundation, either version 3 of the License, or
##  (at your option) any later version.
##
##  This program is distributed in the hope that it will be useful,
##  but WITHOUT ANY WARRANTY; without even the implied warranty of
##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##  GNU Lesser General Public License for more details.
##
##  You should have received a copy of the GNU Lesser General Public License
##  along with this program.  If not, see <http://www.gnu.org/licenses/>.
##------------------------------------------------------------------------------

############################################################################
##                        CPCTELERA ENGINE                                ##
##                 Automatic image conversion file                        ##
##------------------------------------------------------------------------##
## This file is intended for users to automate image conversion from JPG, ##
## PNG, GIF, etc. into C-arrays.                                          ##
##                                                                        ##
## Macro used for conversion is IMG2SPRITES, which has up to 9 parameters:##
##  (1): Image file to be converted into C sprite (PNG, JPG, GIF, etc)    ##
##  (2): Graphics mode (0,1,2) for the generated C sprite                 ##
##  (3): Prefix to add to all C-identifiers generated                     ##
##  (4): Width in pixels of each sprite/tile/etc that will be generated   ##
##  (5): Height in pixels of each sprite/tile/etc that will be generated  ##
##  (6): Firmware palette used to convert the image file into C values    ##
##  (7): (mask / tileset /)                                               ##
##     - "mask":    generate interlaced mask for all sprites converted    ##
##     - "tileset": generate a tileset array with pointers to all sprites ##
##  (8): Output subfolder for generated .C/.H files (in project folder)   ##
##  (9): (hwpalette)                                                      ##
##     - "hwpalette": output palette array with hardware colour values    ##
## (10): Aditional options (you can use this to pass aditional modifiers  ##
##       to cpct_img2tileset)                                             ##
##                                                                        ##
## Macro is used in this way (one line for each image to be converted):   ##
##  $(eval $(call IMG2SPRITES,(1),(2),(3),(4),(5),(6),(7),(8),(9), (10))) ##
##                                                                        ##
## Important:                                                             ##
##  * Do NOT separate macro parameters with spaces, blanks or other chars.##
##    ANY character you put into a macro parameter will be passed to the  ##
##    macro. Therefore ...,src/sprites,... will represent "src/sprites"   ##
##    folder, whereas ...,  src/sprites,... means "  src/sprites" folder. ##
##                                                                        ##
##  * You can omit parameters but leaving them empty. Therefore, if you   ##
##  wanted to specify an output folder but do not want your sprites to    ##
##  have mask and/or tileset, you may omit parameter (7) leaving it empty ##
##     $(eval $(call IMG2SPRITES,imgs/1.png,0,g,4,8,$(PAL),,src/))        ##
############################################################################

## Example firmware palette definition as variable in cpct_img2tileset format
# Blue (non-portal floor and mask color) = 1 
# Bright blue (deactive pellet receptor) = 2
# Red (closed door) = 3
# Magenta (inner cube part) = 4
# Bright Red (active pellet receptor) = 6
# Purple (ground button) = 7
# Cyan (toxic floor) = 10
# Grey (solid color) = 13
# Pastel blue (background color) = 14
# Lime (opened door) = 21
# Pastelm cyan (outter cube part) = 23
# White (floor color) = 26
PALETTE={1 2 3 4 6 7 10 13 14 21 23 26 15 11 16 24}

## Example image conversion
##    This example would convert img/example.png into src/example.{c|h} files.
##    A C-array called pre_example[24*12*2] would be generated with the definition
##    of the image example.png in mode 0 screen pixel format, with interlaced mask.
##    The palette used for conversion is given through the PALETTE variable and
##    a pre_palette[16] array will be generated with the 16 palette colours as
##	  hardware colour values.

#$(eval $(call IMG2SPRITES,assets/ghero.png,0,sprite,8,12,$(PALETTE),,sprites,hwpalette))
$(eval $(call IMG2SPRITES,assets/oportal.png,0,sprite,4,16,$(PALETTE),,src/sprites/,hwpalette))
$(eval $(call IMG2SPRITES,assets/dportal.png,0,sprite,4,16,$(PALETTE),,src/sprites/,))
$(eval $(call IMG2SPRITES,assets/obullet.png,0,sprite,4,4,$(PALETTE),,src/sprites/,))
$(eval $(call IMG2SPRITES,assets/dbullet.png,0,sprite,4,4,$(PALETTE),,src/sprites/,))
$(eval $(call IMG2SPRITES,assets/ghero.png,0,sprite,4,12,$(PALETTE),mask,src/sprites/,))
$(eval $(call IMG2SPRITES,assets/gobstacle.png,0,sprite,4,4,$(PALETTE),,src/sprites/,))
$(eval $(call IMG2SPRITES,assets/blue_portal.png,0,sprite,8,16,$(PALETTE),mask,src/sprites/,))
$(eval $(call IMG2SPRITES,assets/orange_portal.png,0,sprite,8,16,$(PALETTE),mask,src/sprites/,))
$(eval $(call IMG2SPRITES,assets/cake.png,0,sprite,4,8,$(PALETTE),,src/sprites/,))
$(eval $(call IMG2SPRITES,assets/logo.png,0,sprite,60,46,$(PALETTE),,src/sprites/,))
# $(eval $(call IMG2SPRITES,assets/logo.png,0,sprite,120,92,$(PALETTE),,src/sprites/,))

$(eval $(call IMG2SPRITES,assets/cube.png,0,sprite,4,8,$(PALETTE),,src/sprites,))
$(eval $(call IMG2SPRITES,assets/groundButton.png,0,sprite,8,8,$(PALETTE),,src/sprites,))
$(eval $(call IMG2SPRITES,assets/indicator.png,0,sprite,4,4,$(PALETTE),,src/sprites,))
$(eval $(call IMG2SPRITES,assets/pelletReceptor.png,0,sprite,6,12,$(PALETTE),,src/sprites,))
$(eval $(call IMG2SPRITES,assets/pellet2.png,0,sprite,4,8,$(PALETTE),mask,src/sprites,))
$(eval $(call IMG2SPRITES,assets/door.png,0,sprite,16,8,$(PALETTE),,src/sprites,))

$(eval $(call IMG2SPRITES,assets/tiles.png,0,g,4,4,$(PALETTE),tileset,src/sprites,))
