.area _CODE

;;=============================================================================
;; Includes
;;=============================================================================

.include "cpctelera.h.s"
.include "utils.h.s"
.include "constants.h.s"

.include "hero.h.s"

.globl _sprite_door_0
.globl _sprite_door_1

.globl current_level

;;=============================================================================
;; Private data
;;=============================================================================
;; PONER EL PATRN DE DIBUJADO COMO DATO DEL STRUCT?


  defineEntity dr_1, -1, -1, 08, 08, #_sprite_door_0, 0, 0
  defineEntity dr_2, -1, -1, 08, 08, #_sprite_door_0, 0, 0
  defineEntity dr_3, -1, -1, 08, 08, #_sprite_door_0, 0, 0
  defineEntity dr_4, -1, -1, 08, 08, #_sprite_door_0, 0, 0

  NUMBER_OF_DOORS:: .db #3

;;=============================================================================
;; Public functions
;;=============================================================================

;;===========================================================================
;; Unnitializes the doors of a level
;; MODIFIES:
;;  CUBE_DATA => Any atribute of the ground button entities can be modified
;; DESTROYS:
;;  IX
;;===========================================================================
door_uninitialize::
  ld hl, #_sprite_door_0
  set_pos dr_1_data, -1, -1
  ld Ent_state(ix), #0
  ld Ent_spr_l(ix), l
  ld Ent_spr_h(ix), h
  ld Ent_alwaysActive(ix), #0
  set_pos dr_2_data, -1, -1
  ld Ent_state(ix), #0
  ld Ent_spr_l(ix), l
  ld Ent_spr_h(ix), h
  ld Ent_alwaysActive(ix), #0
  set_pos dr_3_data, -1, -1
  ld Ent_state(ix), #0
  ld Ent_spr_l(ix), l
  ld Ent_spr_h(ix), h
  ld Ent_alwaysActive(ix), #0
  set_pos dr_4_data, -1, -1
  ld Ent_state(ix), #0
  ld Ent_spr_l(ix), l
  ld Ent_spr_h(ix), h
  ld Ent_alwaysActive(ix), #0
  ret

;;===========================================================================
;; Initializes the doors of a level
;; MODIFIES:
;;  CUBE_DATA => Any atribute of the ground button entities can be modified
;; DESTROYS:
;;  HL, AF
;;===========================================================================
door_initialize::

  ld  hl, #current_level

  ld  a, #01                  ;;  A=1, compare with level 1
  cp  (hl)                    ;;  |Is A==current_level, if true go to level 1
  jr z, level_1               ;;  \If not, go to next level check

  inc a                       ;;  A++, A=2, compare with level 2
  cp  (hl)
  jr z, level_2

  inc a
  cp  (hl)
  jr z, level_3

  ret

  level_1:
    ld  a, #03
    ld  (NUMBER_OF_DOORS), a

    ld  ix, #dr_1_data

    ld     a, #68
    ld     b, #152
    ld    hl, #_sprite_door_0
    call  utils_initializeCommon
    ld    Ent_state(ix), #00

    ret

  level_2:
  level_3:

  ret

;;===========================================================================
;; Applies a routine to all doors on the level (or first A of them)
;; DESTROYS: AF, BC, DE, HL
;; INPUT:
;;    A => Number of doors to apply the routine (or first A of them)
;;    DE => Address of the routine to apply to the doors
;;===========================================================================
door_applyAll::

  push  de                       ;;  Temporally save DE (routine addr) in stack

  call  door_getPtrHL            ;;  HL=dr_data, start of door array
  ld_ix_hl                       ;;  IX=HL, saves value of HL in IX

  call  door_getOffsetE          ;;  E=door offset, for utils_applyAll
  ld     b, d                    ;;  |B=D, save in B the hgh offset part, BC=DE
  ld     c, e                    ;;  \C=E, save in C the low offset part, BC=DE

  pop   de                       ;;  DE=routineAddr, recover it from  the stack

  ld    a, (NUMBER_OF_DOORS)     ;;  A=04, number of doors to iterate

  call  utils_applyAll           ;;  Apply groundButton_update to all doors
  ret


;;===========================================================================
;; Draws an door
;; INPUT:
;;    IX => door data addr
;; DESTROYS: AF, BC, DE, HL
;;===========================================================================
door_draw::
  ld a, Ent_x(ix)             ;; If x == -1
  cp #-1                      ;;
  ret z                       ;; Return

  ld     a, #00               ;;  A=00, draw sprite without transparencies

  ;ld    ix, #dr_data         ;;  IX=dr_data addr, entity start addr

  ld    hl, #_sprite_door_1  ;; HL=_sprite_door_1, active indc sprite

  ld    Ent_spr_l(ix), l     ;;  EntitySprt=ActiveIndSprite, low part addr
  ld    Ent_spr_h(ix), h     ;;  EntitySprt=ActiveIndSprite, hgh part addr

  call utils_drawSprite       ;;  Draws the sprite given in door entity

  ret

;;===========================================================================
;; Erases an door, permanent entity, it has own color, not paint backgrnd
;; INPUT:
;;    IX => door data addr
;; DESTROYS: AF, BC, DE, HL
;;===========================================================================
door_erase::
  ld a, Ent_x(ix)             ;; If x == -1
  cp #-1                      ;;
  ret z                       ;; Return

  ld     a, #00               ;;  A=00, draw sprite without transparencies

  ;ld    ix, #dr_data         ;;  IX=dr_data addr, entity start addr

  ld    hl, #_sprite_door_0  ;; HL=_sprite_door_0, NoActive ind sprte

  ld    Ent_spr_l(ix), l     ;;  EntitySprt=NoActiveIndSprite, low part addr
  ld    Ent_spr_h(ix), h     ;;  EntitySprt=NoActiveIndSprite, hgh part addr

  call utils_drawSprite       ;;  Draws the sprite given in door entity

  ret

;;===========================================================================
;; Updates an door
;; INPUT:
;;    IX => door data addr
;; DESTROYS: AF, BC, DE, HL
;;===========================================================================
door_update::
  ld a, Ent_x(ix)             ;; If x == -1
  cp #-1                      ;;
  ret z                       ;; Return

  ld     a, Ent_state(ix)     ;;  A=dr_state value, loads the state
  cp    #00                   ;;  |A == 0? If true, it means desactivated so
  jp    z, paint_desactivated ;;  \paint it own bckgrnd, else paint activated

    ;; door is activated
    call    door_draw    ;;  The routine paints it as activated

    ret

  paint_desactivated:
  call    door_erase     ;;  The routine paints it as desactivated

  ret

;;=============================================================================
;; Checks if a door is open (only if collides with hero), used in map.isSolid
;; DESTROYS:
;; RETURNS:
;;  A => If door is open and collides, 0, else 1
;;=============================================================================
door_checkIsOpen::

  ;push bc                ;; Save IX in the stack, recover it at end of routine

  ;; call  door_getPtrHL    ;;  HL=dr_data, start of door array
  ;;
  ;; ld  a, (NUMBER_OF_DOORS); NUMBER OF DOORS IN THE LEVEL
  ;; checkIsOpen_loop:
  ;;   cp  #00                ;; If all doors iterated, no collide, set closed
  ;;   jr  z, door_solid
  ;;
  ;;   push af                ;; Save AF in the stack, between routine calls
  ;;   call  isCollisionDoor ;;  Check if hero or CUBE collides a door
  ;;
  ;;   cp  #00               ;;  A=0x00 if no collides, A=0xFF if collides
  ;;   pop  af                ;; Recover AF from the stack before jump, after cp
  ;;   jr nz, door_found     ;;  If collides, we have found the correct door entity
  ;;
  ;;   call  door_getOffsetE  ;; E=door offset, for adding to HL, move to next door
  ;;   ld  d, #00             ;; D=00, clear before addition
  ;;   add hl, de             ;; HL+=DE, move to next door addr
  ;;
  ;;   dec a                  ;; A--, we have iterated one more door
  ;;
  ;;   jp  checkIsOpen_loop
  ;;
  ;; door_found: ;; REALMENTE CARGAR EN A EL ESTADO DE LA PUERTA
  ;; ;pop bc
  ld_iy_hl

  ld a, Ent_state(iy)  ;; A=state, if 0, means closed, if 1, means open
  dec a                ;; |A--, if it was 1, now it is 0
  cp  #00              ;; \So if now is 0, means that door is open, no solid
  jr  nz, door_solid   ;; If A not 0, means it is closed, so behave as solid

  ld a, #00           ;; A=0, means door behaves like not a solid
  ret


  door_solid:
  ;pop bc
  ld a, #01       ;; A=1, means door behaves like a solid
  ret

;;=============================================================================
;; Computes the distance in bytes (or offset) between two consecutive doors
;; RETURNS:
;;     E => Computed distance between the two doors
;;=============================================================================
door_getOffsetE::
  getEntityOffsetE dr_1, dr_2
  ret

;;=============================================================================
;; Gets a pointer to door dimensions data in HL
;; DESTROYS: HL
;; RETURNS:
;;  HL => Pointer to door data
;;=============================================================================
door_getPtrHL::
  ld     hl, #dr_1_x           ;; Start addr of the struct

  ret

;;=============================================================================
;; Gets the value of the door state
;; INPUT:
;;    IX => door data addr
;; DESTROYS: A
;; RETURNS:
;;  A => Value of the door state
;;=============================================================================
door_getStateA::
  ld     a, Ent_state(ix)     ;;  A=dr_state, put in a the state value

  ret

;;=============================================================================
;; Sets the value of the door state
;; INPUTS:
;;  A => Value of the new state
;;  IX => door data addr
;; MODIFIES:
;;  door_state value => To be updated by the value passed in A
;;=============================================================================
door_setStateA::
    ld    Ent_state(ix), a   ;; dr_state=A, set the new door state

    ret

;;=============================================================================
;; Sets to true (1) a door's alwaysActive state, only if activated by a pltRcpt
;; INPUTS:
;;  IX => door data addr
;; MODIFIES:
;;  door_alwaysActive value => Is set to 1 when pellet recetor is acticated
;;=============================================================================
door_setAlwaysActive::
    ld    Ent_alwaysActive(ix), #01   ;; dr_alwaysActive=01, door open by rcptr

    ret

;;=============================================================================
;; Check if another entity collides with door, entity is passed in HL
;; DESTROYS: A, HL, DE
;; INPUTS:
;;  HL: Pointer to DOOR (DIFFERENT FROM OTHER ISCOLLISION ROUTINES)
;; RETURNS:
;;  A => Collision value, A=0x00 if no collision, A=0xFF if there is collision
;;=============================================================================
isCollisionDoor:
  push  hl                    ;;  Save HL in the stack, we need it for later

  inc   hl                    ;;  HL++, move to Y coord
  dec   (hl)                  ;;  Y--, hitbx start one before, touch is collide
  dec   hl                    ;;  HL--, move to X coord, restore HL

  ex    de, hl                ;;  |Move HL to DE, so we can put in HL
  call  hero_getPtrHL         ;;  \the init of hero dimensions structure

  call  utils_checkCollision  ;;  Check if collide entity and cube
  ld d, a                     ;;  D=A, save returned value from checkCollision

  pop hl                      ;;  Recover HL from stack for use it in checkOpen

  inc   hl                    ;;  HL++, move to Y coord
  inc   (hl)                  ;;  Y++, hitbox is restored
  dec   hl                    ;;  HL--, move to X coord, restore HL
  ld a, d

  ret
