.area _CODE

;;=============================================================================
;; Includes
;;=============================================================================

.include "cpctelera.h.s"
.include "utils.h.s"
.include "constants.h.s"

.include "hero.h.s"           ;;  Hero collisions check
;.include "cube.h.s"           ;;  Cube collisions check
.include "cube_gravity.h.s"
.include "door.h.s"      ;;  Modify state whether collision or not


.globl current_level

;;=============================================================================
;; Private data
;;=============================================================================
;; PONER EL PATRN DE DIBUJADO COMO DATO DEL STRUCT?

;grndButton_data:
;  grndButton_x:        .db #40
;  grndButton_y:        .db #183
;  grndButton_w:        .db #08
;  grndButton_h:        .db #05
;  grndButton_spr:      .dw #_sprite_groundButton

  defineGrnBtn grndButton_1, -1, -1, 04, 08, #_sprite_groundButton, 1
  defineGrnBtn grndButton_2, -1, -1, 04, 08, #_sprite_groundButton, 2
  defineGrnBtn grndButton_3, -1, -1, 04, 08, #_sprite_groundButton, 3
  defineGrnBtn grndButton_4, -1, -1, 04, 08, #_sprite_groundButton, 4

  NUMBER_OF_GRNDBTTNS:: .db #2


;;=============================================================================
;; Public functions
;;=============================================================================

;;===========================================================================
;; Uninitializes the ground buttons
;; MODIFIES:
;;  CUBE_DATA => Any atribute of the ground button entities can be modified
;; DESTROYS:
;;  IX
;;===========================================================================
groundButton_uninitialize::
  set_pos grndButton_1_data, -1, -1
  set_pos grndButton_2_data, -1, -1
  set_pos grndButton_3_data, -1, -1
  set_pos grndButton_4_data, -1, -1
  ret

;;===========================================================================
;; Initializes the ground buttons of a level
;; MODIFIES:
;;  CUBE_DATA => Any atribute of the ground button entities can be modified
;; DESTROYS:
;;  HL, AF
;;===========================================================================
groundButton_initialize::

  ld  hl, #current_level

  ld  a, #01                  ;;  A=1, compare with level 1
  cp  (hl)                    ;;  |Is A==current_level, if true go to level 1
  jr z, level_1               ;;  \If not, go to next level check

  inc a                       ;;  A++, A=2, compare with level 2
  cp  (hl)
  jr z, level_2

  inc a
  cp  (hl)
  jr z, level_3

  ret

  level_1:
    ld  a, #02
    ld  (NUMBER_OF_GRNDBTTNS), a

    ld  ix, #grndButton_1_data

    ld     a, #05
    ld     b, #183
    ld    hl, #_sprite_groundButton
    call  utils_initializeCommon
    ;;  Which indicator is connected to, is constant
    ;;  So it will not change on any level, so do not need to initialize it
    ret

  level_2:
  level_3:

  ret

;;===========================================================================
;; Applies a routine to all groundButtons on the level (or first A of them)
;; DESTROYS: AF, BC, DE, HL
;; INPUT:
;;    A => Number of groundButtons to apply the routine (or first A of them)
;;    DE => Address of the routine to apply to the groundButtons
;;===========================================================================
groundButton_applyAll::

  push  de                       ;;  Temporally save DE (routine addr) in stack

  call  groundButton_getPtrHL    ;;  HL=hero_data, start of hero array
  ld_ix_hl                       ;;  IX=HL, saves value of HL in IX

  call  groundButton_getOffsetE  ;;  E=grndButton offset, for utils_applyAll
  ld     b, d                    ;;  |B=D, save in B the hgh offset part, BC=DE
  ld     c, e                    ;;  \C=E, save in C the low offset part, BC=DE

  pop   de                       ;;  DE=routineAddr, recover it from  the stack

  ld    a, (NUMBER_OF_GRNDBTTNS) ;;  A=04, number of groundButtons to iterate

  call  utils_applyAll           ;;  Apply groundButton_update to all grndbttns
  ret

;;===========================================================================
;; Draws a ground button
;; INPUT:
;;    IX => groundButton data addr
;; DESTROYS: AF, BC, DE, HL
;;===========================================================================
groundButton_draw::
  ld a, Ent_x(ix)             ;; If x == -1
  cp #-1                      ;;
  ret z                       ;; Return

  ld     a, #00               ;;  A=00, draw sprite without transparencies

  inc   Ent_y(ix)             ;;  GrndButton_y value++, draw 1 below hitbox

  ;ld   ix, #grndButton_1_data;;  IX=groundButton_data addr, entity start addr
  call utils_drawSprite       ;;  Draws the sprite given in grndButton entity

  dec   Ent_y(ix)             ;;  GrndButton_y value--, restore original value

  ret

;;===========================================================================
;; Erases a ground button
;; INPUT:
;;    IX => groundButton data addr
;; DESTROYS: AF, BC, DE, HL
;;===========================================================================
groundButton_erase::
  ld a, Ent_x(ix)             ;; If x == -1
  cp #-1                      ;;
  ret z                       ;; Return

  ld     a, #00               ;;  A=00, draw sprite without transparencies

  inc   Ent_y(ix)             ;;  GrndButton_y value++, draw 1 below hitbox

  ;ld   ix, #grndButton_1_data;;  IX=groundButton_data addr, entity start addr
  call utils_drawSprite       ;;  Draws the sprite given in grndButton entity

  dec   Ent_y(ix)             ;;  GrndButton_y value--, restore original value

  ret

;;===========================================================================
;; Updates a ground button
;; DESTROYS: A, HL, DE
;; MODIFIES:
;;  Ind_state value => To be updated by the value passed in A
;;===========================================================================
groundButton_update::
  ld a, Ent_x(ix)             ;; If x == -1
  cp #-1                      ;;
  ret z                       ;; Return


  call  hero_getPtrHL         ;;  Set HL to init of hero dimensions structure
  call  isCollisionGrndBttn   ;;  See if hero collide grbttn, A=0 FALSE, 1 TRUE

  cp    #0xFF                 ;;  |If collisn, activate indicator, ind_state=1
  jp      z, collision        ;;  \If no collisn, desactivate ind, ind_state=0

    ;; There is no hero collision, check if there is cube collision
    ld_hl_ix                      ;;  HL=IX, pass as paramtr for next routine call
    call  cube_gravity_isCollision ;;  Check if A BUTTON collides with ANY cube

    cp    #00                 ;;  |If collisn, activate indicator, ind_state=1
    jp     nz, collision      ;;  \If no collisn, desactivate ind, ind_state=0


      ;;  There is no cube collisn, then zero collisns, not activate indicator
      ld     a, #0x00           ;;  A=0, to set indicator state to 0, not active

      ;ld__d_ixh
      ;ld__e_ixl
      push  ix                ;;  Save IX temporally in the stack

      ld    h, Ent_grndBttnInd_h(ix)  ;;  H=indctrAddr, high value part loaded
      ld    l, Ent_grndBttnInd_l(ix)  ;;  L=indctrAddr, low value part loaded
      ld_ix_hl

      ld a, Ent_alwaysActive(ix)
      cp #01
      jr z, not_modify_state ;; State is always active due to receptor
        ;call  indicator_setStateA;  Sets the state to 0, means desactivated
        ld    Ent_state(ix), a   ;; Door_state=A, set the new door state
      not_modify_state:
      ;ld__ixh_d
      ;ld__ixl_e
      pop  ix                 ;;  Recover IX from the stack

      ret

  collision:
    ld     a, #01             ;;  A=1, to set indicator state to 1, is active

    ;ld__d_ixh
    ;ld__e_ixl
    push  ix                ;;  Save IX temporally in the stack

    ld    h, Ent_grndBttnInd_h(ix)  ;;  H=indctrAddr, high value part loaded
    ld    l, Ent_grndBttnInd_l(ix)  ;;  L=indctrAddr, low value part loaded
    ld_ix_hl

    ld    Ent_state(ix), a   ;; Ind_state=A, set the new indicator state
    ;call  indicator_setStateA ;;  Sets the state to 1, means activated

    ;ld__ixh_d
    ;ld__ixl_e
    pop  ix                 ;;  Recover IX from the stack

  ret

;;=============================================================================
;; Computes the distance in bytes (or offset) between two consecutive grndBttns
;; RETURNS:
;;     E => Computed distance between the two groundButtons
;;=============================================================================
groundButton_getOffsetE::
  getEntityOffsetE grndButton_1, grndButton_2
  ret

;;=============================================================================
;; Gets a pointer to ground button dimensions data in HL
;; DESTROYS: HL
;; RETURNS:
;;  HL => Pointer to ground button data
;;=============================================================================
groundButton_getPtrHL::
  ld hl, #grndButton_1_x     ;; Start addr of the struct
  ;;NO ME HACE FALTA UN PUNTERO EN HL, TENGO IX APUNTANDO AL GRNDBUTTON ACTUAL
  ret

;;=============================================================================
;; Private functions
;;=============================================================================

;;=============================================================================
;; Check if another entity collides with ground button, entity is passed in HL
;; DESTROYS: A, HL, DE
;; INPUTS:
;;  HL: Pointer to begin of entity, check if the entity collides with grndbtton
;; RETURNS:
;;  A => Collision value, A=0x00 if collision, A=0xFF if there is no collision
;;=============================================================================
isCollisionGrndBttn:
  ex    de, hl                ;;  |Move HL to DE, so we can put in HL IX
  ld_hl_ix                    ;;  \HL=IX, HL has now beginning of grndbttn addr

  ;call  groundButton_getPtrHL ;;  \the init of grnd button dimensions structure

  call  utils_checkCollision  ;;  Check if collide entity and grndButton

  ret
