.area _CODE

.include "cpctelera.h.s"
.include "hero.h.s"
.include "cube_gravity.h.s"
.include "constants.h.s"

;;========================================
;; Private variables
;;========================================

;; position gravity gun (x, y, h, w) same as heroe
gun_gravity_state:    .db #-1             ;; -1 (not activated gun), 1 (activated gun)
gun_gravity_range:    .db #20

;;========================================
;; Public methods
;;========================================

gun_gravity_update::
  call check_user_input
  ret

gun_gravity_state_getPtrA::
  ld a, (gun_gravity_state)
  ret
;;========================================
;; Private methods
;;========================================

;;=========================================================
;; checking the keys of keyboard
;; G activaded: gravity field
;;=========================================================
check_user_input:
 call cpct_scanKeyboard_asm
 ld hl, #Key_G
 call cpct_isKeyPressed_asm
 cp #0
 jr z, desactive_gfield
 ;; key G pressed
 call hero_getPtrHL
 call cube_gravity_getPtrDE
 call gun_gravity_attract_object
 ret
 desactive_gfield:
 ld a, #-1
 ld (gun_gravity_state), a
 call cube_gravity_change_state     ;; moves free
 ret


;;===============================================================
;; Check cube_gravity is in the gravity field generated by the gun
;; INPUTS: de address cube_gravity, hl address hero
;;==============================================================
;; check grange order check axis (higher speed)?
gun_gravity_attract_object:
  ;; AXIS X: cube_gravity is behind gravity field
  ;; if (obs_x + obs_w <= hero_x - gun_gravity_range)
  call gravity_formula              ;; de = cube_gravity_x, hl = hero_x
  jr z, no_gravity
  jp m, no_gravity

  ;; AXIS X: cube_gravity is ahead gravity field
  ;; if (hero_x + hero_w <= obs_x - gun_gravity_range)
  dec de                            ;; de points cube_gravity_y
  dec de                            ;; de points cube_gravity_x
  ex de, hl                         ;; swap
  call gravity_formula              ;; de = hero_x, hl = cube_gravity_x
  jr z, no_gravity
  jp m, no_gravity

  ;; AXIS Y: cube_gravity is above gravity field
  ;; if (hero_y + hero_h <= obs_y - gun_gravity_range)
  dec de                            ;; de points cube_gravity_y
  inc hl                            ;; hl points hero_y
  ex de, hl                         ;; swap
  call gravity_formula              ;; de = cube_gravity_y, hl = hero_y
  jr z, no_gravity
  jp m, no_gravity
  ;; AXIS Y: de is above hl
  ;; if (hero_y + hero_h <= obs_y - gun_gravity_range
  dec de                            ;; de points cube_gravity_w
  dec de                            ;; de points cube_gravity_y
  ex de, hl                         ;; swap
  call gravity_formula              ;; de = cube_gravity_x, hl = hero_x
  jr z, no_gravity
  jp m, no_gravity
  ;; gravity acts
  ld a, #1
  ld (gun_gravity_state), a
  call cube_gravity_change_state       ;; moves like hero
  ;;ld a, #0xFF
  ret
  no_gravity:
  ;;ld a, #0x0
  ret

;;====================================================
;; gravity distance between gun and object
;; INPUTS: de (cube_gravity) hl (gun)
;; (obs_x + obs_w - hero_x + gun_gravity_range <= 0)
;;====================================================
gravity_formula:
  ld a, (de)          ;; cube_gravity_x
  ld c, a             ;; C = cube_gravity_x
  inc de              ;; de points cube_gravity_y
  inc de              ;; de points cube_gravity_w
  ld a, (de)          ;; A = cube_gravity_w
  add c               ;; obs_w + obs_x
  sub (hl)            ;; A - value direction hl (whose value is hero_x
  ld b, a
  ld a, (gun_gravity_range)
  ld c, a
  ld a, b
  add c               ;; gravity range
  ret
