.area _CODE

;;=============================================================================
;; Includes
;;=============================================================================
.include "cpctelera.h.s"
.include "constants.h.s"
.include "utils.h.s"

.include "portals.h.s"
.include "cube_gravity.h.s"
.include "door.h.s"

.globl _g_tileset             ;;  Tilset to use
.globl _map1                  ;;  First map to use
;; .globl _map2                  ;;  Second map to use
;; .globl _map3                  ;;  Third map to use
.globl _map10

check_cube:: .db #1

lvls:
    .dw #_map1 ;;, #_map2

current_map::
    .dw #0x0000

;; solid_tiles:
;;     .db #0, #08, #09, #10, #12, #11, #12, #13, #14, #15, #16, #17, #18, #19, #80
;;
;; no_portal_tiles:
;;     .db #2, #04, #05, #06, #07, #31, #32, #33, #34, #35, #36, #37, #39, #39, #80
;;
;; poison_tiles:
;;     .db #3, #20, #21, #22, #21, #22, #23, #24, #25, #26, #27, #28, #29, #30, #80


tile_type::
    .db #Solid

defineEntity aux_entity, 0, 0, 0, 0, 0, 0, 0


;;=============================================================================
;; Private data
;;=============================================================================

;;=============================================================================
;; Public functions
;;=============================================================================


;;===========================================================================
;; Initializes the new tileset
;; DESTROYS: HL
;;===========================================================================
map_initialize::
  ld  hl, #_g_tileset         ;;  HL=_g_tileset addr, beginning of tilset array
  call cpct_etm_setTileset2x4_asm ;;  Set the new tileset to use

  ret

;;===========================================================================
;; Draws the map in #current_map
;; DESTROYS: AF, BC, DE, HL AF', BC', DE', HL'
;;===========================================================================
map_draw::
  ld  hl, #_map1              ;;  HL=_map1 addr, begin. of map1 tileIndex array
  ;; ld  hl, #_map2              ;;  HL=_map1 addr, begin. of map1 tileIndex array
  push  hl                    ;;  Load HL into the stack for map routine

  ld  hl, #0xC000             ;; HL=0xC000, vid memry ptr, tilemap in video mem
  push  hl                    ;;  Load HL into the stack for map routine

  ld  bc, #0000               ;;  BC=0000, coord in map to start to draw (x, y)
  ld   d, #50               ;;  D=32, height to draw (not map1 one), is 0x20
  ld   e, #40               ;;  E=80, width to draw (not map1 one) is 0x50
  ;ld  de, #0x2028             ;;  DE=0x2050 (32, 50), works only if hex
  ld   a, #40                 ;;  A=80, width of map in dec, numbr of tiles
  call  cpct_etm_drawTileBox2x4_asm ;; Draw part of map, acording to params

  ret

;;===========================================================================
;; Draws the map in #current_map
;; DESTROYS: AF, BC, DE, HL AF', BC', DE', HL'
;;===========================================================================
map_printLevel::
    ld hl, (current_map)
    push  hl                            ;; BC pushed to the stack
    ld hl, #0xC000                      ;; HL = vmem start
    push hl                             ;; HL pushed to the stack

    ld bc, #00                          ;; C = 0, B = 0 (start of tilemap)
    ld de, #0x3228                      ;; E = 40, D = 50 (w and h of tilemap)
    ld a, #40                           ;; A = 40 (w of a full tilemap row)
    call cpct_etm_drawTileBox2x4_asm    ;; Draws all the level

    ret

;;
;; Sets the current level
;; PARAMETERS:
;;   - A: Level number
;; DESTROYS:
;;   - HL, BC
map_setLevel::
    ld hl, #lvls            ;; HL -> *lvls
    ld c, a                 ;; |
    ld b, #0                ;; \ BC = offset
    add hl, bc              ;; | HL = *_level[current_map]
    add hl, bc              ;; \ (each map has 2 bytes)
    ld c, (hl)              ;; |
    inc hl                  ;; |
    ld b, (hl)              ;; |
    ld h, b                 ;; |
    ld l, c                 ;; \ HL = *current_map
    ld (current_map), hl  ;; current_leve = A
    ret

;;
;; Paints the screen where an object was
;;
map_drawTilesIn::
    ld hl, #current_map               ;; HL = **current_map
    ld e, (hl)                          ;; |
    inc hl                              ;; |
    ld d, (hl)                          ;; \ DE = *current_map
    push de
    ld hl, #0xC000                      ;; HL = vmem start
    push hl                             ;; HL pushed to the stack
    ld hl, #0

    ld de, #0x0403

    ld a, #40
    call cpct_etm_drawTileBox2x4_asm
    ret

;;
;; Returns the tile in x, y
;; PARAMETERS:
;;   - C: x
;;   - B: y
getTileIn::
    srl c   ;; C = C / 2
    srl b   ;; |
    srl b   ;; \ B = B / 4
    ret


;;
;; Returns the coordinates of tile x, y
;; PARAMETERS:
;;   - C: tile_x
;;   - B: tile_y
;; RETURNS:
;;   - C: x
;;   - B: y
getTileOf::
    sla c   ;; C = C * 2
    sla b   ;; |
    sla b   ;; \ B = B * 4
    ret


;;
;; Checks if tiles above entity are solid
;; PARAMETERS:
;;   - IX: Entity
;; RETURNS:
;;   - A:
;; DESTROYS:
;;   - BC, DE, HL
map_checkUpperBounds::
    ;; Up-left corner
    ld c, Ent_x(ix)
    ld b, Ent_y(ix)
    dec b

    push bc                 ;; Save tile coords in stack for isSolid check
    call getTileIn

    call getTile            ;; A = tile_under_entity
    ld c, a                 ;; C = tile_under_entity
    pop de                  ;; Get tile coords from stack for isSolid check
    push de
    call isSolid            ;; If it is solid, DownBound = 1
    pop bc

    cp #0
    jr z, up_left_bound_false

    ld a, Ent_w(ix)
    add Ent_h(ix)
    cp #5
    jr z, up_bound_true

    ld a, #Up
    ld d, a
    call portals_portalOnTile
    cp #0
    jr z, up_bound_true

    up_left_bound_false:

    ;; Up-right corner
    ld a, Ent_x(ix)
    add a, Ent_w(ix)
    dec a
    ld c, a
    ld b, Ent_y(ix)
    dec b
    push bc                 ;; Save tile coords in stack for isSolid check

    call getTileIn

    call getTile            ;; A = tile_under_entity
    ld c, a                 ;; C = tile_under_entity
    pop de                  ;; Get tile coords from stack for isSolid check
    push de
    call isSolid            ;; If it is solid, DownBound = 1
    pop bc

    cp #0
    jr z, up_right_bound_false

    ld a, Ent_w(ix)
    add Ent_h(ix)
    cp #5
    jr z, up_bound_true

    ld a, #Up
    ld d, a
    call portals_portalOnTile
    cp #0
    jr z, up_bound_true

    up_right_bound_false:
        ld a, #0
    ret

    up_bound_true:
        ld a, #1
    ret

;;
;; Checks if tiles on the left entity are solid
;; PARAMETERS:
;;   - IX: Entity
;; RETURNS:
;;   - B: bitwise bounds
;; DESTROYS:
;;   - A, BC, DE, HL
map_checkLeftBounds::
    ;; Up-left corner
    ld c, Ent_x(ix)
    dec c
    ld b, Ent_y(ix)
    push bc                 ;; tile coordinate saved
    call getTileIn
    ;;push bc                 ;; Save tile coords in stack for isSolid check

    call getTile            ;; A = tile_under_entity
    ld c, a                 ;; C = tile_under_entity
    ;pop de                  ;; Get tile coords from stack for isSolid check
    pop de
    push de
    call isSolid            ;; If it is solid, DownBound = 1
    pop bc                  ;; tile coordinate rescued

    cp #0                       ;; If tile is not solid:
    jr z, left_up_bound_false   ;;   right bound false

    ld a, Ent_w(ix)
    add Ent_h(ix)
    cp #5
    jr z, left_bound_true

    ld a, #Left                 ;; |
    ld d, a                     ;; \ orientation = right
    call portals_portalOnTile   ;; portalOnTile(BC, D)
    cp #0                       ;; |
    jr z, left_bound_true       ;; \ if there is not portal, right_bound = true

    left_up_bound_false:

    ;; Down-right corner
    ld c, Ent_x(ix)
    dec c
    ld a, Ent_y(ix)
    add a, Ent_h(ix)
    dec a
    ld b, a
    push bc                 ;; tile coordinate saved
    call getTileIn
    ;;push bc                 ;; Save tile coords in stack for isSolid check

    call getTile            ;; A = tile_under_entity
    ld c, a                 ;; C = tile_under_entity
    ;pop de                  ;; Get tile coords from stack for isSolid check
    pop de
    push de
    call isSolid            ;; If it is solid, DownBound = 1
    pop bc                  ;; tile coordinate rescued

    cp #0                   ;; If tile is not solid:
    jr z, left_bound_false  ;; Return false

    ld a, Ent_w(ix)
    add Ent_h(ix)
    cp #5
    jr z, left_bound_true

    ld a, #Left                 ;; |
    ld d, a                     ;; \ orientation = right
    call portals_portalOnTile   ;; portalOnTile(BC, D)
    cp #0                       ;; |
    jr z, left_bound_true       ;; \ if there is not portal, right_bound = true

    left_bound_false:
    ld a, #0
    ret

    left_bound_true:
        ld a, #1
    ret

;;
;; Checks if tiles on the right entity are solid
;; PARAMETERS:
;;   - IX: Entity
;; RETURNS:
;;   - B: bitwise bounds
;; DESTROYS:
;;   - A, BC, DE, HL
map_checkRightBounds::
    ;; Up-right corner
    ld a, Ent_x(ix)
    add a, Ent_w(ix)
    ld c, a
    ld b, Ent_y(ix)
    push bc                 ;; tile coordinate saved
    call getTileIn
    ;;push bc                 ;; Save tile coords in stack for isSolid check

    call getTile            ;; A = tile_under_entity
    ld c, a                 ;; C = tile_under_entity
    pop de                  ;; Get tile coords from stack for isSolid check
    push de
    call isSolid            ;; If it is solid, DownBound = 1
    pop bc                      ;; tile coordinate rescued

    cp #0                       ;; If tile is not solid:
    jr z, right_up_bound_false  ;;   right bound false

    ld a, Ent_w(ix)
    add Ent_h(ix)
    cp #5
    jr z, right_bound_true

    ld a, #Right                ;; |
    ld d, a                     ;; \ orientation = right
    call portals_portalOnTile   ;; portalOnTile(BC, D)
    cp #0                       ;; |
    jr z, right_bound_true      ;; \ if there is not portal, right_bound = true

    right_up_bound_false:

    ;; Down-right corner
    ld a, Ent_x(ix)
    add a, Ent_w(ix)
    ld c, a
    ld a, Ent_y(ix)
    add a, Ent_h(ix)
    dec a
    ld b, a
    push bc                 ;; tile coordinate saved
    call getTileIn
    ;;push bc                 ;; Save tile coords in stack for isSolid check

    ;; pop bc
    ;; call isSolid

    call getTile            ;; A = tile_under_entity
    ld c, a                 ;; C = tile_under_entity
    pop de                  ;; Get tile coords from stack for isSolid check
    push de
    call isSolid            ;; If it is solid, DownBound = 1
    pop bc                      ;; tile coordinate rescued

    cp #0                   ;; If tile is not solid:
    jr z, right_bound_false ;; Return false

    ld a, Ent_w(ix)
    add Ent_h(ix)
    cp #5
    jr z, right_bound_true


    ld a, #Right                ;; |
    ld d, a                     ;; \ orientation = right
    call portals_portalOnTile   ;; portalOnTile(BC, D)
    cp #0                       ;; |
    jr z, right_bound_true      ;; \ if there is not portal, right_bound = true

    right_bound_false:
    ld a, #0
    ret

    right_bound_true:
        ld a, #1
    ret

;;
;; Checks if tiles under entity are solid
;; PARAMETERS:
;;   - IX: Entity
;; RETURNS:
;;   - B: bitwise bounds
;; DESTROYS:
;;   - A, BC, DE, HL
map_checkLowerBounds::
    ;; Down-left corner
    ld c, Ent_x(ix)
    ld a, Ent_y(ix)
    add a, Ent_h(ix)
    ld b, a
    push bc                 ;; tile coordinate saved
    call getTileIn

    call getTile            ;; A = tile_under_entity
    ld c, a                 ;; C = tile_under_entity
    pop de
    push de
    call isSolid            ;; If it is solid, DownBound = 1
    pop bc                  ;; tile coordinate rescued

    cp #0
    jr z, down_left_bound_false

    ld a, Ent_w(ix)
    add Ent_h(ix)
    cp #5
    jr z, down_bound_true

    ld a, #Down                 ;; |
    ld d, a                     ;; \ orientation = down
    call portals_portalOnTile   ;; portalOnTile(BC, D)
    cp #0                       ;; |
    jr z, down_bound_true       ;; \ if there is not portal, down_bound = true

    down_left_bound_false:

    ;; Down-right corner
    ld a, Ent_x(ix)
    add a, Ent_w(ix)
    dec a
    ld c, a
    ld a, Ent_y(ix)
    add a, Ent_h(ix)
    ld b, a
    push bc                 ;; tile coordinate saved
    call getTileIn

    call getTile            ;; A = tile_under_entity
    ld c, a                 ;; C = tile_under_entity
    pop de
    push de
    call isSolid            ;; If it is solid, DownBound = 1
    pop bc                  ;; tile coordinate rescued

    cp #0
    jr z, down_bound_false

    ld a, Ent_w(ix)
    add Ent_h(ix)
    cp #5
    jr z, down_bound_true

    ld a, #Down                 ;; |
    ld d, a                     ;; \ orientation = down
    call portals_portalOnTile   ;; portalOnTile(BC, D)
    cp #0                       ;; |
    jr z, down_bound_true       ;; \ if there is not portal, down_bound = true

    down_bound_false:
        ld a, #0
    ret

    down_bound_true:
        ld a, #1
    ret

;;
;; PRIVATE PART
;;


;;
;; Gets the tile in x, y coordinates
;; PARAMETERS:
;;   - C: x
;;   - B: y
;; RETURNS:
;;   - A: Tile
getTile:
    push hl

    ld hl, #0           ;; Sets hl counter to 0
    inc b               ;; |
    djnz row_loop       ;; \ if b != 0, increase hl row
    end_row:

    ld b, c             ;; B = C
    inc b               ;; |
    djnz col_loop       ;; \ if b != 0, increase hl col
    end_col:

    ex de, hl                   ;; |
    ;; call map_getCurrentLevel ;; |
    ;; ld h, b                     ;; |
    ;; ld l, c                     ;; |
    ld hl, (current_map)
    add hl, de                  ;; |
    ld a, (hl)                  ;; \ a = tile

    pop hl

    ret

    row_loop:
        ;; ld h, b
        ;; ld l, c
        ;; call quickMul28
        ld de, #40          ;; DE = row_size
        add hl, de          ;; HL += row_size
        djnz row_loop       ;; if --b != 0, goto row_loop
        jr end_row

    col_loop:
        inc hl              ;; HL++
        djnz col_loop       ;; if --b != 0, goto col_loop
        jr end_col

;;
;; Multiplies HL per 0x28
;; PARAMETERS:
;;   - HL
;; DESTROYS:
;;   - DE
quickMul28:
    ld d, h         ;; |
    ld e, l         ;; \ DE = HL
    ld h, l         ;; |
    ld l, #0        ;; \ HL *= 0x10
    sla h           ;; HL *= 0x2
    add hl, de
    add hl, de
    add hl, de
    add hl, de
    add hl, de
    add hl, de
    add hl, de
    add hl, de
    ret


inArray:
  ld a, (hl)
  cp c
  jr z, in_array_true
  cp #80
  jr z, in_array_false
  inc hl
  jp inArray

  in_array_true:
    ld a, #1
    ret
  in_array_false:
    ld a, #0
    ret

;; C is between L and H
isBetween:
  ;; C >= L && C <= H
  ;; L < C && H > C
  ld a, c
  cp l
  jp m, isBetween_false
  ld a, h
  cp c
  jp m, isBetween_false
  jr isBetween_true

  isBetween_false:
    ld a, #0
    ret
  isBetween_true:
    ld a, #1
    ret

;;
;; Checks if tile is solid
;; PARAMETERS:
;;   - C: Tile
;;   - D: Tile Y
;;   - E: Tile X
;; RETURNS:
;;   - A: isSolid (0/1)
isSolid:
    ld a, #Solid
    ld (tile_type), a

    ;; ld hl, #solid_tiles         ;; HL -> solid_tiles
    ;; call inArray
    ;; cp #1
    ;; ret z
    ;;
    ld l, #8
    ld h, #19
    call isBetween
    cp #1
    ret z
    ld a, c
    cp #0
    jr z, it_is_solid

    ld l, #20
    ld h, #31
    call isBetween
    cp #1
    jr nz, no_poisonous_tile
        ld a, #Poison
        ld (tile_type), a
        jr it_is_solid
    no_poisonous_tile:
    ld a, c
    cp #3
    jr z, it_is_solid


    ld l, #32
    ld h, #40
    call isBetween
    cp #1
    jr nz, no_nportal_tile
        ld a, #NoPortal
        ld (tile_type), a
        jr it_is_solid
    no_nportal_tile:

    ld l, #2
    ld h, #8
    call isBetween
    cp #1
    jr nz, no_nportal_tile2
        ld a, #NoPortal
        ld (tile_type), a
        jr it_is_solid
    no_nportal_tile2:
    ld a, c
    cp #2
    jr z, it_is_solid


    push de
    call isCube
    cp #01
    pop de
    jr z, it_is_solid

    call isDoor                ;; If value returned in A is 1, is door
    cp #01
    jr z, is_open_door         ;; Is door, go to check if is opened or closed

    ld a, #00
    ret

    it_is_solid:
    ld a, #01
    ret

    is_open_door:
    ld a, #NoPortal
    ld (tile_type), a

    call  door_checkIsOpen   ;; Sets A to 0 if it is open, to 1 if not (solid)
    ret

;;
;; Checks if tile is door
;; PARAMETERS:
;;   - C: Tile
;; RETURNS:
;;   - A: isDoor (0/1)
isDoor:
    ;; ld hl, #door_tiles         ;; HL -> solid_tiles
    ;;
    ;; isDoor_loop:
    ;;     ld a, (hl)
    ;;     cp c
    ;;     jr z, it_is_door       ;; If C in array, it is door
    ;;     cp #80                  ;; |
    ;;     jr z, it_is_not_door   ;; \ If it isn't door it reaches #80 value in array
    ;;     inc hl
    ;;     jp isDoor_loop
    ld iy, #aux_entity_data
    ld Ent_x(iy), e
    ld Ent_y(iy), d
    ld Ent_h(iy), #1
    ld Ent_w(iy), #1
    ld de, #aux_entity_data
    ld hl, #dr_1_data
    call utils_checkCollision
    cp #0
    jr nz, it_is_door
    ld hl, #dr_2_data
    call utils_checkCollision
     cp #0
     jr nz, it_is_door
     ld hl, #dr_3_data
     call utils_checkCollision
     cp #0
     jr nz, it_is_door
     ld hl, #dr_4_data
     call utils_checkCollision
    cp #0
    jr nz, it_is_door

    it_is_not_door:
    ld a, #0
    ret
    it_is_door:
    ld a, #1
    ret


;;
;; Checks if tile is cube
;;
isCube:
    ld a, (check_cube)
    cp #0
    ret z

    ld iy, #aux_entity_data
    ;; sla d
    ;; sla d
    ;; sla e
    ld Ent_x(iy), e
    ld Ent_y(iy), d
    ld Ent_w(iy), #1
    ld Ent_h(iy), #1
    ld hl, #aux_entity_data
    ld de, #cube_gravity_data
    call utils_checkCollision
    cp #0
    ret z

    ld a, #1
    ret
;;
;; Checks if door is open
;; PARAMETERS:
;;   - C: Tile
;; RETURNS:
;;   - A: isDoor (0/1)
isDoorOpen:
    ret

;;
;; Returns a pointer to the current level
;; RETURNS:
;;   - HL: *current_map
map_getCurrentLevel:
    ld hl, (current_map)             ;; HL = **current_map
    ;; ld c, (hl)                          ;; |
    ;; inc hl                              ;; |
    ;; ld b, (hl)                          ;; \ BC = *current_map
    ret
