.include "constants.h.s"
.include "map.h.s"
.include "utils.h.s"
.globl _sprite_blue_portal_0
.globl _sprite_blue_portal_1
.globl _sprite_blue_portal_2
.globl _sprite_blue_portal_3
.globl _sprite_orange_portal_0
.globl _sprite_orange_portal_1
.globl _sprite_orange_portal_2
.globl _sprite_orange_portal_3

;; defineEntity blue ...
;; defineEntity orange ...
defineEntity orange_portal, -1, -1, 4, 16, #_sprite_orange_portal_0, 0, 0
defineEntity blue_portal, -1, -1, 4, 16, #_sprite_blue_portal_0, 0, 0

orange_portal_o: .db #0
blue_portal_o: .db #0

;; ===================================================
;; ===================================================
;; PUBLIC FUNCTIONS
;; ===================================================
;; ===================================================

;;
;; Sets the blue portal where the bullet collide
;; PARAMETERS:
;;   - IX: Bullet
;;   - A: Portal orientation
portals_setBluePortal::
    ld iy, #blue_portal_data
    ld (blue_portal_o), a
    call setPortal

    ld a, (blue_portal_o)
    cp #Up
    jr nz, blue_not_up
        ld hl, #_sprite_blue_portal_2
        ld Ent_spr_h(iy), h
        ld Ent_spr_l(iy), l
        ret
    blue_not_up:
    cp #Right
    jr nz, blue_not_right
        ld hl, #_sprite_blue_portal_3
        ld Ent_spr_h(iy), h
        ld Ent_spr_l(iy), l
        ret
    blue_not_right:
    cp #Down
    jr nz, blue_not_down
        ld hl, #_sprite_blue_portal_0
        ld Ent_spr_h(iy), h
        ld Ent_spr_l(iy), l
        ret
    blue_not_down:
        ld hl, #_sprite_blue_portal_1
        ld Ent_spr_h(iy), h
        ld Ent_spr_l(iy), l
        ret
    ret

;;
;; Sets the orange portal where the bullet collide
;; PARAMETERS:
;;   - IX: Bullet
;;   - A: Portal orientation
portals_setOrangePortal::
    ld iy, #orange_portal_data
    ld (orange_portal_o), a
    call setPortal

    ld a, (orange_portal_o)
    cp #Up
    jr nz, orange_not_up
        ld hl, #_sprite_orange_portal_2
        ld Ent_spr_h(iy), h
        ld Ent_spr_l(iy), l
        ret
    orange_not_up:
    cp #Right
    jr nz, orange_not_right
        ld hl, #_sprite_orange_portal_3
        ld Ent_spr_h(iy), h
        ld Ent_spr_l(iy), l
        ret
    orange_not_right:
    cp #Down
    jr nz, orange_not_down
        ld hl, #_sprite_orange_portal_0
        ld Ent_spr_h(iy), h
        ld Ent_spr_l(iy), l
        ret
    orange_not_down:
        ld hl, #_sprite_orange_portal_1
        ld Ent_spr_h(iy), h
        ld Ent_spr_l(iy), l
        ret
    ret

;;
;; Erases the portal
portals_erase::
    push ix
    ld ix, #blue_portal_data    ;; IX = *portal_data
    ld a, Ent_x(ix)             ;; |
    cp #-1                      ;; \ If portal_x == -1
    jr z, do_not_erase_blue      ;;     don't erase blue
        ld c, Ent_x(ix)             ;; C = portal_x
        ld b, Ent_y(ix)             ;; B = portal_y
        call getTileIn              ;; BC = tile x, y
        call map_drawTilesIn        ;; Redraw tiles in BC

    do_not_erase_blue:

    ld ix, #orange_portal_data  ;; IX = *portal_data
    ld a, Ent_x(ix)             ;; |
    cp #-1                      ;; \ If portal_x == -1
    pop ix
    ret z                       ;;     don't erase orange(just return)
        push ix
        ld ix, #orange_portal_data  ;; IX = *portal_data
        ld c, Ent_x(ix)             ;; C = portal_x
        ld b, Ent_y(ix)             ;; B = portal_y
        call getTileIn              ;; BC = tile x, y
        call map_drawTilesIn        ;; Redraw tiles in BC
        pop ix
    ret

portals_update::
    ret


;;
;; Draws the portals. If a portal does not exist (x == -1),
;;   it obiously doesn't draw it
;; DESTROYS:
;;   - AF, BC, DE, HL, IX

portals_draw::
    ld ix, #blue_portal_data    ;; IX = *portal_data
    ld a, Ent_x(ix)             ;; |
    cp #-1                      ;; \ If portal_x == -1
    jr z, do_not_draw_blue      ;;     don't draw blue
        ld a, #1
        call utils_drawSprite   ;; else: draw it

    do_not_draw_blue:

    ld ix, #orange_portal_data  ;; IX = *portal_data
    ld a, Ent_x(ix)             ;; |
    cp #-1                      ;; \ If portal_x == -1
    ret z                       ;;     don't draw orange (just return)
        ld a, #1
        call utils_drawSprite   ;; else: draw it

    ret


;;
;; Chacks if in certain position is there portal
;; PARAMETERS:
;;   - C: x
;;   - B: y
;;   - D: orientation
;; RETURNS:
;;   - A: True/False (1/0)
portals_portalOnTile::
    ld iy, #blue_portal_data
    ld a, Ent_x(iy)
    cp #-1
    ld a, #0
    ret z

    ld iy, #orange_portal_data
    ld a, Ent_x(iy)
    cp #-1
    ld a, #0
    ret z

    call getTileIn      ;; |
    call getTileOf      ;; \ we get x, y of the first pixel of the tile in bc


    ld a, (blue_portal_o)
    push de
    cp d                    ;; |
    jr nz, check_orange     ;; \ If blue not that orientation, don't check it

        ld hl, #blue_portal_data        ;;
        call portals_singlePortalOnTile ;;
        cp #1                           ;;
        pop de
        ret z                           ;; \ If onBlue, return true
        push de

    check_orange:

    pop de
    ld a, (orange_portal_o)
    cp d                    ;; | on this orientation:
    jr nz, onTile_false     ;; \    return false

        ld hl, #orange_portal_data
        call portals_singlePortalOnTile
        ret

    onTile_false:           ;; |
        ld a, #0            ;; \ Return false
    ret

portals_uninitialize::
    set_pos orange_portal_data, -1, -1
    set_pos blue_portal_data, -1, -1
    ret

;;
;; Cross the portal if the hero is inside
;; PARAMETERS
;;   - IX: hero
portals_crossIfInside::
    ld iy, #orange_portal_data
    ld a, Ent_x(iy)
    cp #-1
    ret z
    ld iy, #blue_portal_data
    ld a, Ent_x(iy)
    cp #-1
    ret z
    ;; blue portal
    call portals_isInside
    cp #1
    jr z, it_is_inside_blue


    ;; orange portal
    ld iy, #orange_portal_data
    call portals_isInside
    cp #1
    jr z, it_is_inside_orange

    ld a, #-1

    ret

    it_is_inside_blue:
        ld c, Ent_x(ix)                 ;; |
        ld b, Ent_y(ix)                 ;; |
        call getTileIn                  ;; |
        call map_drawTilesIn            ;; \ Erase the hero
        ld a, (orange_portal_o)
        ld iy, #orange_portal_data
        call place_hero
    ret

    it_is_inside_orange:
        ld c, Ent_x(ix)                 ;; |
        ld b, Ent_y(ix)                 ;; |
        call getTileIn                  ;; |
        call map_drawTilesIn            ;; \ Erase the hero
        ld a, (blue_portal_o)
        ld iy, #blue_portal_data
        call place_hero
    ret


;; ===================================================
;; ===================================================
;; PRIVATE PART
;; ===================================================
;; ===================================================


;;
;; Sets a portal
;; PARAMETERS:
;;   - IX: Bullet
;;   - IY: Portal
;;   - A: orientation
setPortal:
    push af

    call portals_erase

    ld a, Ent_x(ix)     ;; |
    ld Ent_x(iy), a     ;; \ portal_x = bullet_x

    ld a, Ent_y(ix)     ;; |
    ld Ent_y(iy), a     ;; \ portal_x = bullet_x

    pop af

    cp #Down
    jr z, set_to_down
    cp #Right
    jr z, set_to_right
    cp #Left
    jr z, set_to_left
    cp #Up
    jr z, set_to_up

    set_to_down:
        ld a, Ent_y(iy)     ;; |
        add Ent_h(ix)       ;; |
        ld Ent_y(iy), a     ;; \ portal_y += bullet_h

        ld a, Ent_x(iy)     ;; |
        ld b, Ent_w(ix)     ;; |
        srl b               ;; | portal center set
        add b               ;; | to bullet center
        ld b, Ent_w(iy)     ;; |
        srl b               ;; |
        sub b               ;; |
        ld Ent_x(iy), a     ;; \ portal_x += bullet_w / 2 - portal_w / 2

        srl Ent_x(iy)
        sla Ent_x(iy)
    ret

    set_to_up:
        ld a, Ent_y(iy)     ;; |
        sub Ent_h(iy)       ;; |
        ld Ent_y(iy), a     ;; \ portal_y -= portal_h

        ld a, Ent_x(iy)     ;; |
        ld b, Ent_w(ix)     ;; |
        srl b               ;; | portal center set
        add b               ;; | to bullet center
        ld b, Ent_w(iy)     ;; |
        srl b               ;; |
        sub b               ;; |
        ld Ent_x(iy), a     ;; \ portal_x += bullet_w / 2 - portal_w / 2

        srl Ent_x(iy)
        sla Ent_x(iy)
    ret

    set_to_right:
        ld a, Ent_x(iy)     ;; |
        add Ent_w(ix)       ;; |
        ld Ent_x(iy), a     ;; \ portal_x += bullet_w

        ld a, Ent_y(iy)     ;; |
        ld b, Ent_h(ix)     ;; |
        srl b               ;; | portal center set
        add b               ;; | to bullet center
        ld b, Ent_h(iy)     ;; |
        srl b               ;; |
        sub b               ;; |
        res 0, a            ;;   |
        res 1, a            ;;   \ Round portal_y to tile
        ld Ent_y(iy), a     ;; \ portal_y += bullet_h / 2 - portal_h / 2
    ret

    set_to_left:
        ld a, Ent_x(iy)     ;; |
        sub Ent_w(iy)       ;; |
        ld Ent_x(iy), a     ;; \ portal_x += portal_h

        ld a, Ent_y(iy)     ;; |
        ld b, Ent_h(ix)     ;; |
        srl b               ;; | portal center set
        add b               ;; | to bullet center
        ld b, Ent_h(iy)     ;; |
        srl b               ;; |
        sub b               ;; |
        res 0, a            ;;   |
        res 1, a            ;;   \ Round portal_y to tile
        ld Ent_y(iy), a     ;; \ portal_y += bullet_h / 2 - portal_h / 2
    ret

;;
;; Places the hero in a portal

;; PARAMTERS:
;;   - IX: Hero
;;   - IY: Portal
;;   - A: Portal orientation
place_hero:
    cp #Up
    jr z, in_up_portal

    cp #Down
    jr z, in_down_portal

    cp #Right
    jr z, in_right_portal

    cp #Left
    jr z, in_left_portal
    ret

    in_up_portal:
        ld a, Ent_x(iy)     ;;
        ld b, a             ;; B = portal_x
        ld a, Ent_w(iy)     ;; A = portal_w
        srl a               ;; A = portal_w / 2
        add b               ;; A = portal_x + portal_w / 2
        ld b, a             ;; B = portal_x + portal_w / 2
        ld a, Ent_w(ix)     ;; A = hero_w
        srl a               ;; A = hero_w / 2
        ld c, a             ;; C = hero_w / 2
        ld a, b             ;; A = portal_x + portal_w / 2
        sub c               ;; A = A = portal_x + portal_w / 2 - hero_w / 2
        ld Ent_x(ix), a     ;; hero_x = A = portal_x + portal_w / 2 - hero_w / 2

        ld a, Ent_y(iy)     ;; |
        add a, Ent_h(iy)    ;; |
        ld Ent_y(ix), a     ;; \ hero_y = portal_y + portal_h
        ld a, #Up
        ret

    in_down_portal:
        ld a, Ent_x(iy)     ;;
        ld b, a             ;; B = portal_x
        ld a, Ent_w(iy)     ;; A = portal_w
        srl a               ;; A = portal_w / 2
        add b               ;; A = portal_x + portal_w / 2
        ld b, a             ;; B = portal_x + portal_w / 2
        ld a, Ent_w(ix)     ;; A = hero_w
        srl a               ;; A = hero_w / 2
        ld c, a             ;; C = hero_w / 2
        ld a, b             ;; A = portal_x + portal_w / 2
        sub c               ;; A = A = portal_x + portal_w / 2 - hero_w / 2
        ld Ent_x(ix), a     ;; hero_x = A = portal_x + portal_w / 2 - hero_w / 2

        ld a, Ent_y(iy)     ;; |
        sub a, Ent_h(ix)    ;; |
        ld Ent_y(ix), a     ;; \ hero_y = portal_y - portal_h - hero_h
        ld a, #Down
        ret

    in_right_portal:
        ld a, Ent_x(iy)     ;; |
        sub a, Ent_w(ix)    ;; |
        ld Ent_x(ix), a     ;; \ hero_x = portal_x - hero_w

        ld a, Ent_y(iy)     ;; |
        add a, Ent_h(iy)    ;; |
        sub a, Ent_h(ix)    ;; |
        ld Ent_y(ix), a     ;; \ hero_y + portal_h = portal_y + portal_h (floor aligned)

        ld Ent_vy(ix), #0   ;; Reset hero_vy
        ld a, #Right
        ret

    in_left_portal:
        ld a, Ent_x(iy)     ;; |
        add a, Ent_w(iy)    ;; |
        ld Ent_x(ix), a     ;; \ hero_x = portal_x - portal_w

        ld a, Ent_y(iy)     ;; |
        add a, Ent_h(iy)    ;; |
        sub a, Ent_h(ix)    ;; |
        ld Ent_y(ix), a     ;; \ hero_y + portal_h = portal_y + portal_h (floor aligned)

        ld Ent_vy(ix), #0   ;; Reset hero_vy
        ld a, #Left
        ret



;;
;; Checks if the hero is inside a portal
;; PARAMETERS:
;;   - IX: hero
;;   - IY: portal
portals_isInside:
    ;; hero_x >= portal_x and hero_x + w <= portal_y + w
    
    ;; if portax_x > hero_x
    ;;   return false
    ld a, Ent_x(ix)
    cp Ent_x(iy)
    jr c, not_inside

    ;; if hero_x + hero_x > portal_x + portal_w
    ;;   return false
    ld a, Ent_x(ix)
    add a, Ent_w(ix)
    ld b, a
    ld a, Ent_x(iy)
    add a, Ent_w(iy)
    cp b
    jr c, not_inside


    ld a, Ent_y(ix)
    cp Ent_y(iy)
    jr c, not_inside

    ld a, Ent_y(ix)
    add a, Ent_h(ix)
    ld b, a
    ld a, Ent_y(iy)
    add a, Ent_h(iy)
    cp b
    jr c, not_inside

    ld a, #1
    ret

    not_inside:
    ld a, #0
    ret

;;
;; Checks if a certain portal is on a tile
;; PARAMETERS:
;;   - HL: Portal
;;   - C: tile_x
;;   - B: tile_y
;; DESTROYS:
;;   - HL, DE
;; RETURNS:
;;   - A: true/false
portals_singlePortalOnTile:
    ld iy, #aux_entity_data
    ld Ent_x(iy), c
    ld Ent_y(iy), b
    ld Ent_w(iy), #1
    ld Ent_h(iy), #1
    ld de, #aux_entity_data
    ;; ld de, #0
    push bc
    call utils_checkCollision
    pop bc
    cp #0                       ;; If checkCollision == 0
    ret z                       ;; return 0

    ld a, #1                    ;; else
    ret                         ;; return 1
