;;  KEY FIRMWARE CODES

.equ Key_W            ,#0x0807  ;;  Move up (jump)
.equ Key_A            ,#0x2008  ;;  Move left
;;.equ Key_S          ,#0x1007  ;;  Move down
.equ Key_D            ,#0x2007  ;;  Move right

.equ Key_G            ,#0x1006  ;;  Activate gravity gun

.equ Key_I            ,#0x0804  ;;  Aim up portal gun
.equ Key_K            ,#0x2004  ;;  Aim down portal gun
.equ Key_J            ,#0x2005  ;;  Aim left portal gun
.equ Key_L            ,#0x1004  ;;  Aim right portal gun

.equ Key_P            ,#0x0803  ;;  Change portal
.equ Key_O            ,#0x0404  ;;  Change portal

.equ Key_Del          ,#0x8009  ;;  Emergency key for resetting level
.equ Key_Esc          ,#0x0408
.equ Key_1			,#0x0108
.equ Key_2			,#0x0208
.equ Key_3			,#0x0207

.equ JoyUp		,#0x0109
.equ JoyLeft		,#0x0409
.equ JoyDown		,#0x0209
.equ JoyRight		,#0x0809
.equ JoyFire1		,#0x1009
.equ JoyFire2		,#0x2009

.equ Dir_IJ            ,1  ;;  Aim up-left portal gun
.equ Dir_I             ,2  ;;  Aim up portal gun
.equ Dir_IL            ,3  ;;  Aim up-right portal gun
.equ Dir_J             ,4  ;;  Aim left portal gun
.equ Dir_L             ,5  ;;  Aim right portal gun
.equ Dir_KJ            ,6  ;;  Aim down-left portal gun
.equ Dir_K             ,7  ;;  Aim down portal gun
.equ Dir_KL            ,8  ;;  Aim down-right portal gun

.equ Solid, 0
.equ NoPortal, 1
.equ Poison, 2

.equ PoisonTile, 14
.equ NoPortalTile, 7

.equ bg_color, 0x03
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

.macro ld__d_ixh
   .dw #0x54DD         ;; ld d, ixh
.endm

.macro ld__ixh_d
   .dw #0x62DD         ;; ld ixh, d
.endm

.macro ld__ixl_e
   .dw #0x6BDD         ;; ld ixl, e
.endm

.macro ld__e_ixl
   .dw #0x5DDD         ;; ld e, ixl
.endm
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
.macro ld__iyh_d
   .dw #0x62FD         ;; ld iyh, d
.endm

.macro ld__iyl_e
   .dw #0x6BFD         ;; ld iyl, e
.endm
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


.macro defineTimer name
    name'_timer::
        name'_m: .dw 0x0000
        name'_s: .dw 0x0000
        name'_d: .dw 0x0000
.endm

;;
;; Entity Macro
;;
.macro defineEntity name, x, y, w, h, spr, velX, velY
    name'_data::
        name'_x: .db x
        name'_y: .db y
        name'_w: .db w
        name'_h: .db h
        name'_sprite: .dw spr
        name'_velX: .db velX
        name'_velY: .db velY
        name'_jump: .db #-1
.endm

.equ Ent_x, 0
.equ Ent_y, 1
.equ Ent_w, 2
.equ Ent_h, 3
.equ Ent_spr_l, 4  ;; IN L IS ADDRESS BEGIN AS USE LITTLE ENDIAN, L FOR LOW
.equ Ent_spr_h, 5  ;; IN H IS ADDRESS END AS USE LITTLE ENDIAN, H FOR HIGH
.equ Ent_vx, 6
.equ Ent_vy, 7
.equ Ent_jmp, 8

.equ Ent_grndBttnInd_l, 6
.equ Ent_grndBttnInd_h, 7


;; Setter macro
.macro set_pos data, x, y
    ld ix, #data
    ld Ent_x(ix), #x
    ld Ent_y(ix), #y
.endm

.macro set_vel_ix velX, velY
    ld Ent_vx(ix), #velX
    ld Ent_vy(ix), #velY
.endm

;; FOR AN ENTITY POINTING TO ITS STATE IN IX
.equ Ent_state, 6
.equ Ent_pltState, 8

;; FOR INDICATORS AND PELLET RECEPTORS POINTING TO ITS DOOR ADDRESS
.equ Ent_indDoor_l, 7
.equ Ent_indDoor_h, 8

.equ Ent_pltRecpDoor_l, 7
.equ Ent_pltRecpDoor_h, 8

;; FOR PELLET RECEPTOR TO INDICATE TO AN ACTIVATOR TO NOT DESACTIVATE A DOOR
.equ Ent_alwaysActive, 7

;; FOR PELLET SENSE OF MOVEMENT
.equ Ent_sense, 7

;; FOR GROUND BUTTON HAVING CORRESPONDING INDICATOR
.macro defineGrnBtn name, x, y, w, h, spr, id
    name'_data::
        name'_x: .db x
        name'_y: .db y
        name'_w: .db w
        name'_h: .db h
        name'_sprite: .dw spr
        name'_ind: .dw dr_'id'_data
.endm

.macro defineIndicator name, x, y, w, h, spr, state, id
name'_data::
    name'_x: .db x
    name'_y: .db y
    name'_w: .db w
    name'_h: .db h
    name'_sprite: .dw spr
    name'_state: .db state
    name'_doorId: .dw #dr_'id'_data
.endm

;;
;; Entity Macro
;;
.macro definePellet name, x, y, w, h, spr, velX, velY, state
    name'_data::
        name'_x: .db x
        name'_y: .db y
        name'_w: .db w
        name'_h: .db h
        name'_sprite: .dw spr
        name'_velX: .db velX
        name'_velY: .db velY
        name'_state: .db state
.endm

;;
;; Orientation constants
;;
.equ Up, 0
.equ Right, 1
.equ Down, 2
.equ Left, 3

;;  ANIMATION ORIGIN FRAMES (first frame of animation, so to reset it):
.globl _sprite_cube          ;; Address of cube's animation start, its first frame
.globl _sprite_groundButton  ;; Address of grndButton's animation start, its first frame
;; .globl _sprite_pellet        ;; Address of pellet's animation start, its first frame
.globl _sprite_pellet2       ;; Address of pellet's animation start, its first frame

.globl _sprite_indicator_0   ;; Address of indicator_notActive animation start, its first frame
.globl _sprite_indicator_1   ;; Address of indicator_active animation start, its first frame

.globl _sprite_pelletReceptor_0 ;; Address of pelletReceptor_notActive animation start, its first frame
.globl _sprite_pelletReceptor_1 ;; Address of pelletReceptor_active animation start, its first frame


;============================================================================
;============================================================================
;============================================================================

;;  GET ENTITY OFFSETS, FOR ITERATING ARRAYS
.macro getEntityOffsetE entity1, entity2
  ld    e, #entity2'_data - entity1'_data
.endm

.macro getEntityOffsetC entity1, entity2
  ld    c, #entity2'_data - entity1'_data
.endm

;; To save in IX, IY some values, pass 2byt between HL and IX, IY and viceversa
.equ ix_value, 42
.equ iy_value, 42

;;  MOVES HL VALUE IN IX
.macro ld_ix_hl
  push hl
  pop ix

.endm

.macro ld_hl_ix
  push ix
  pop  hl
.endm


;;  MOVES HL VALUE IN IY
.macro ld_iy_hl
  ld (iy_value), hl
  ld iy, (iy_value)
.endm

.macro ld_hl_iy
  ld  (iy_value), iy
  ld  hl, (iy_value)
.endm
