.area _CODE

.include "cpctelera.h.s"
.include "constants.h.s"         ;; global constants used among the game
.include "utils.h.s"
.include "map.h.s"
.include "pellet.h.s"
.include "portals.h.s"
.include "level_manager.h.s"
.include "cube_gravity.h.s"
.include "gun_gravity.h.s"
.globl _sprite_ghero_0
.globl _sprite_ghero_1

;;========================================
;; Private variables
;;========================================

;; hero_x:    .db #34
;; hero_y:    .db #172
;; hero_w:    .db #2
;; hero_h:    .db #12
;; hero_jump: .db #-1               ;; contains the index of jumptable

defineEntity hero, 40, 80, 2, 12, #_sprite_ghero_1, 0, 0
hero_sense:: .db #1                 ;; 1 (moves right), -1 (moves left)

hero_fall: .db #-1

hero_rpush: .db #0
hero_lpush: .db #0

hero_deaths:: .dw #0x0000

;; ;; Jump Table
;; jumptable:
;;   .db #-3, #-2, #-1, #-1          ;;up 8 and down 8
;;   .db #-1, #00, #00, #00
;;   .db #01, #02, #02, #03
;;   .db #0x80                      ;; 80hex = 128dec, ends jump

push_incrementation:
    .db #4
;; Push table
;;   For simulating decelerations to hero (jumps, pushings...)
pushtable:
    ;; .db #04
    ;; .db #04
    ;; .db #04
    ;; .db #04
    .db #02
    .db #01
    .db #01
    .db #00
    .db #00
    .db #80

;; Falltable
;;   For simulating accelerations to hero (just falls)
falltable::
    .db #01
    .db #02
    .db #02
    .db #04
    .db #80

;;========================================
;;========================================
;; Public functions
;;========================================
;;========================================

;; hero_erase::
;;   sub a
;;   call drawHero
;;   ret
;;
;; hero_draw::
;;   ld a, #0xFF
;;   call drawHero
;;   ret

hero_uninitialize::
  set_pos hero_data, -1, -1
  set_vel_ix 0, 0
  sub a
  ld (hero_rpush), a
  ld (hero_lpush), a
  ret

hero_erase::
    ld ix, #hero_data
    ld c, Ent_x(ix)
    ld b, Ent_y(ix)
    ;; call getTileIn
    ;; call map_drawTilesIn
    ld de, #0xC000
    call cpct_getScreenPtr_asm
    ex de, hl
    ld a, #bg_color
    ld c, Ent_w(ix)
    ld b, Ent_h(ix)
    jp    cpct_drawSolidBox_asm
    
hero_draw::
    ld a, #01
    ld ix, #hero_data
    jp    utils_drawSprite    ;; \ Draws the hero    

hero_move::
    ld ix, #hero_data
    call utils_moveEntity

    ld a, (hero_fall)
    inc a
    ret nz

    ld a, (hero_jump)
    inc a
    ret nz

    srl Ent_y(ix)
    srl Ent_y(ix)
    sla Ent_y(ix)
    sla Ent_y(ix)
    ret

hero_kill::
    ld hl, #level_attempts
    inc (hl)
    jp    levelManager_gameOver
    
;;===========================================================================
;; Updates the hero
;; DESTROYS: AT LEST: AF, BC, HL
;;===========================================================================
hero_update::
  call checkUserInput

  ;; ld a, (game_over)
  ;; cp a, #1
  ;; ret z

 ;; push af
 ;;  ld a, #8
 ;;    ;; cp #1
 ;;    ;; jr nz, no_h_poison_collision
 ;;    ;; call hero_kill
 ;;    ;; ret
 ;;  pop af

  no_h_poison_collision:

  call hero_pushControl
  call jumpControl
  call heroFall
  call checkHeroStillAlive

  call portals_crossIfInside
  cp #-1
  ret z


  ld ix, #hero_data
  ld Ent_vx(ix), #0
  ld Ent_vy(ix), #0

  cp #Up
  ret z
  
  no_cross_to_up:

  ld hl, #hero_fall
  ld (hl), #-1

  cp #Left
  jp z, hero_pushRight    
  
  cp #Right
  jp z, hero_pushLeft 

  cp #Down
  ret nz
  jp   startJump
  
;;========================================
;; Private functions
;;========================================

;;=====================================================
;; Say in the next moviment exist cube gravity
;; INPUT: A=-1 (move left), A=1 (move right)
;; OUTPUT: A=1 (EXIST collision) A=0 (NOT EXIST collision)
;;=====================================================
hero_check_collision_cube_gravity:
  ;; check collision gravity cube
  ld b, a                   ;; B = -1 (left) || 1 (right)
  ld hl, #hero_data         ;; hl -> hero_x
  ld a, (hl)                ;; A = hero_x
  ld c, a                   ;; C = hero_x (use after)
  ld a, (hl)                ;; A = hero_x
  add b                     ;; A = hero_x + sense_hero_x
  ld (hl), a                ;; pellet_x += sense_hero_x
  ld de, #cube_gravity_data
  call utils_checkCollision
  or a
  ld a, c                   ;; A = hero_x
  ld (hl), a                ;; previous value hero_x
  jr nz, exist_collision
  sub a
  ret
  exist_collision:
  ld a, #1
  ret

;;========================================
;; Move hero one position to right
;; Destroys: a
;; Input: hl
;;========================================
moveHeroRight:
  ld ix, #hero_data
  ld hl, #_sprite_ghero_1
  ld Ent_spr_h(ix), h
  ld Ent_spr_l(ix), l
  call map_checkRightBounds    ;; A = isSomethingAtRight
  dec a                        ;; |
  ld a, (tile_type)
  jr nz, do_move_right
  
  cp #Poison
  call z, hero_kill				;stack overflow? (jr z, hero_kill)
  ret

  do_move_right:

  cp #Poison
  call z, hero_kill

  ;; ld ix, #hero_data           ;; IX destroyed before


  inc Ent_vx(ix)              ;; Increments vx (moves right)
  ret

;;========================================
;; Move hero one position to left
;; Destroys: a
;; Input: hl
;;========================================
moveHeroLeft:
  ld ix, #hero_data
  ld hl, #_sprite_ghero_0
  ld Ent_spr_h(ix), h
  ld Ent_spr_l(ix), l
  call map_checkLeftBounds     ;; A = isSomethingAtRight
  dec a                           ;; |
  jr nz, do_move_left

  ld a, (tile_type)
  cp #Poison
  call z, hero_kill
  ret

  do_move_left:

  ;; ld ix, #hero_data           ;; IX destroyed before
  dec Ent_vx(ix)              ;; Decrements vx (moves left)
  ret


;;========================================
;; star jump with key W of keyboard
;; Destroys:
;; Input:
;;========================================
startJump:
  ld a, (hero_jump)           ;; Loads hero_jump to A
  inc a                       ;; Checks if jump is active
  ret nz                      ;; Returns if condition

  ld a, (hero_fall)           ;; Loads hero_jump to A
  inc a                       ;; |
  ret nz                      ;; \ If the hero is falling, don't jump

  sub a                       ;; |
  ld (hero_jump), a           ;; \ Sets hero_jump to first position

  ret


;;========================================
;; Simulates gravity to hero
;; Destroys: IX, BC, HL, A
;; Input:
;;========================================
heroFall:
  ld ix, #hero_data       ;; ix = *hero_data
  ld a, Ent_jmp(ix)       ;; Loads hero_jump to A
  inc a                   ;; |
  ret nz                  ;; \ If the hero is jumping, don't fall

  ld a, Ent_x(ix)
  add Ent_vx(ix)
  ld Ent_x(ix), a
  call map_checkLowerBounds
  push af

  ld ix, #hero_data
  ld a, Ent_x(ix)
  sub Ent_vx(ix)
  ld Ent_x(ix), a

  pop af
  dec a			;cp #1

  jr z, do_not_fall

  ;; Get Fall Value
  ld a, (hero_fall)       ;; Loads to A the index of current fall
  cp #-1                  ;; |
  jr nz, already_falling  ;; \ If hero_fall == -1, set to 0

    sub a                 ;; |
    ld (hero_fall), a       ;; \ hero_fall = 0

  already_falling:
  ;; inc a                   ;; Increases the index in A
  ld hl, #falltable           ;; HL points to start of fall
  ld c, a                     ;; |
  ld b, #0                    ;; \ BC = A (offset)
  add hl, bc                  ;; HL += BC
  ld a, (hl)                  ;; A = fall[hero_fall]
  cp a, #80                   ;; |
  ld a, #4
  jr z, last_hero_fall_pos    ;; \ If A is not last value, increase fall vel

  ld a, (hero_fall)
  inc a
  ld (hero_fall), a

  ld c, a                     ;; Load the fall index of A in C
  ld b, #0                    ;; BC = A(offset)
  ld hl, #falltable           ;; HL -> *pushtable
  ld a, (hl)                  ;; A = fall[hero_fall]

  ;; dec a                    ;; Decreases the index in A

  last_hero_fall_pos:
  ;; ld (hero_fall), a        ;; Save the the hero_fall

  cp #4                       ;;
  push af
  jr nz, do_not_increment     ;;


  ld hl, #hero_data
  ld de, #blue_portal_data
  call utils_checkCollision
  or a
  jr nz, do_not_increment

  ld hl, #hero_data
  ld de, #orange_portal_data
  call utils_checkCollision
  or a
  jr nz, do_not_increment

  do_increment:

  ld hl, #push_incrementation
  inc (hl)

  do_not_increment:

  pop af

  ld ix, #hero_data
  ld Ent_vy(ix), a            ;; loads fall value in vel_y

  ret

  do_not_fall:
    ld a, (tile_type)
    cp #Poison
    call z, hero_kill

    ld ix, #hero_data
    ld hl, #hero_fall       ;; |
    ld (hl), #-1            ;; \ Set hero_fall variable to -1 (not falling)
    ld hl, #push_incrementation
    ld (hl), #4
  ret                     ;; And return

hero_pushRight:
  ld a, (push_incrementation)
  ld (hero_rpush), a
  ret

hero_pushLeft:
  ld a, (push_incrementation)
  ld (hero_lpush), a
  ret

hero_pushControl:
  ld a, (hero_rpush)
  or a
  jr z, check_push_left

  ld ix, #hero_data
  call map_checkRightBounds
  dec a			;cp #1
  jr nz, do_push_right

    sub a
    ld (hero_rpush), a
    ret

  do_push_right:

  ld ix, #hero_data
  inc Ent_vx(ix)

  ld a, (hero_rpush)
  dec a
  ld (hero_rpush), a


  check_push_left:

  ld a, (hero_lpush)
  or a
  ret z

  ld ix, #hero_data
  call map_checkLeftBounds
  dec a			;cp #1
  jr nz, do_push_left

    sub a
    ld (hero_lpush), a
    ret

  do_push_left:

  ld ix, #hero_data
  dec Ent_vx(ix)

  ld a, (hero_lpush)
  dec a
  ld (hero_lpush), a

  ret

;;========================================
;; Check the index of jumptable and end when
;; arrive las posistion array, (start, continue and end) jump
;; Destroys: a, b, c
;; Input: hero_jump
;;========================================
jumpControl:
  ld a, (hero_jump)
  inc a			;cp #-1
  ret z

  ld ix, #hero_data

  call map_checkUpperBounds
  dec a			;cp #1
  jr z, finish_jump

  call check_jump_inc
  dec a			;cp #1
  ret z

  ld a, (hero_jump)
  ld hl, #pushtable
  ld c, a
  ld b, #0
  add hl, bc
  ld a, (hl)

  cp #80
  jr z, finish_jump

  ld a, Ent_vy(ix)
  sub (hl)
  ld Ent_vy(ix), a

  ld hl, #hero_jump
  inc (hl)

  ret

  finish_jump:
    ld a, (tile_type)
    cp #Poison
    call z, hero_kill
    ld a, #-1
    ld (hero_jump), a
    ;ld a, #-1
    ld (push_incrementation), a
  ret

;; jumpControl:
;;   ld ix, #hero_data       ;; ix = *hero_data
;;   ld a, (hero_jump)       ;; A = hero_jump status
;;   cp #-1                  ;; Checks if it's jumpig (a=!-1)
;;   ret z                   ;; If A is 0 => return and not jump
;;
;;   ;; Get Jump Value
;;   ld hl, #pushtable       ;; HL points to start of jumptable
;;   ld c, a                 ;; |
;;   ld b, #0                ;; \ BC = A (offset)
;;   add hl, bc              ;; HL += BC
;;
;;   ;; Do the Jump Movement
;;   push hl
;;   call map_checkUpperBounds    ;; A = isSomethingOnTop
;;   pop hl
;;   cp #1                        ;; |
;;   jr z, finish_jump            ;; \ if it is, end jump
;;
;;   ld a, Ent_vy(ix)        ;; A = actual vel
;;   sub a, (hl)             ;; A += jump_movement
;;   ld Ent_vy(ix), a        ;; vel_y = jumpMovement
;;
;;   call check_jump_inc
;;   cp #1
;;   ret z
;;
;;   ;; Increment hero_jump index
;;   inc hl                  ;; Increases pushtable pointer
;;   ld a, (hl)              ;; Loads falltable value to A
;;   cp #80                  ;; Checks if last pos (0x80)
;;   jr z, finish_jump       ;; And continues if it is not
;;
;;   ld a, (hero_jump)       ;; Load hero_jump to A
;;   inc a                   ;; |
;;   ld (hero_jump), a       ;; \ hero_jump++
;;   ret
;;
;;   finish_jump:
;;     ld a, #-1           ;; |
;;     ld (hero_jump), a   ;; \ Finishes jump (hero_jump = -1)
;;     ld a, #-3            ;; push_table.length + fall steps in portal
;;     ld (push_incrementation), a
;;   ret

check_jump_inc:
  ld a, (push_incrementation)
  or a
  ret z
    dec a
    ld (push_incrementation), a
    ld a, Ent_vy(ix)
    sub a, #4
    ld Ent_vy(ix), a
    ld a, #1
    ret
  

;;========================================
;; Check user input and destroys
;; Destroys:
;;========================================
checkUserInput:
  ld ix, #hero_data
  ld Ent_vx(ix), #0               ;; |
  ld Ent_vy(ix), #0               ;; \ Ent_vel set up to 0

  call cpct_scanKeyboard_asm    ;; scan all keys of the keyboard
                               ;; after checks if a concrete key is pressed or not
  ld hl, #JoyFire2
  call cpct_isKeyPressed_asm		;no movement if Fire2 is pressed
  or a
  ret nz

  ld hl, #JoyRight              ;; store in the register hl Key D
  call cpct_isKeyPressed_asm    ;; if in a concrete key the value of flag z is !0
                                ;; the key is pressed
                                ;; sub #5:  substract 5 to acumulator register A
                                ;; so value register A changes
  or a                          ;; substract 0 to register A
                                ;; flag z value is actived the value of substraction is zero
                                ;; however, the vaule of A is not changed, conserves the value previous substraction
  jr z, key_a
  call moveHeroRight            ;; use: hl | destroys: hl, a
  key_a:
  ld hl, #JoyLeft
  call cpct_isKeyPressed_asm    ;; if in a concrete key is pressed the value of flag is !0
  or a
  jr z, key_w
  call moveHeroLeft             ;; use: hl | destroys: a
  key_w:
  ld hl, #JoyUp                 ;; store register hl Key D
  call cpct_isKeyPressed_asm    ;; if in a concrete key is pressed the value of flag is !0
  or a
  jr z, key_del
  call startJump
  key_del:
  ld hl, #Key_Del               ;;  Reset the level, in case of emergency
  call cpct_isKeyPressed_asm    ;; if in a concrete key is pressed the value of flag is !0
  or a
  ret z
  jp hero_kill
  


 ;;===========================================================================
 ;;===========================================================================
 ;;===========================================================================
 ;;===========================================================================
 ;;===========================================================================

;;============================================================================
;; Check if hero is still alive (death by energy ball, poisonous floor...)
;; DESTROYS:  A, HL, DE
;; MODIFIES:
;;    Hero_x => Can set it in its respawn X coordinate if destroyed
;;    Hero_y => Can set it in its respawn Y coordinate if destroyed
;;============================================================================
checkHeroStillAlive:
  ld hl, #hero_data
  call pellet_isCollisionAll
  or a
  jp nz, hero_kill         	 ;;  Sets the hero coordinates to the respawn ones

  ret                         ;;  If there is no collision do nothing

  