.area _CODE

;;=============================================================================
;; Includes
;;=============================================================================

.include "door.h.s"

.include "cpctelera.h.s"
.include "utils.h.s"
.include "constants.h.s"

.globl current_level

;;=============================================================================
;; Private data
;;=============================================================================
;; PONER EL PATRN DE DIBUJADO COMO DATO DEL STRUCT?
  defineIndicator ind_1, -1, -1, 02, 04, #_sprite_indicator_0, 0, 1
  defineIndicator ind_2, -1, -1, 02, 04, #_sprite_indicator_0, 0, 2
  defineIndicator ind_3, -1, -1, 02, 04, #_sprite_indicator_0, 0, 3
  defineIndicator ind_4, -1, -1, 02, 04, #_sprite_indicator_0, 0, 4

  NUMBER_OF_INDICATORS:: .db #1

;;=============================================================================
;; Public functions
;;=============================================================================

;;===========================================================================
;; Initializes the indicators of a level
;; MODIFIES:
;;  CUBE_DATA => Any atribute of the ground button entities can be modified
;; DESTROYS:
;;  HL, AF
;;===========================================================================
indicator_uninitialize::
  ld hl, #_sprite_indicator_0
  ;ld a,#-1
  set_pos ind_1_data, -1, -1
  ld Ent_spr_l(ix), l
  ld Ent_spr_h(ix), h
  set_pos ind_2_data, -1, -1
  ld Ent_spr_l(ix), l
  ld Ent_spr_h(ix), h
  set_pos ind_3_data, -1, -1
  ld Ent_spr_l(ix), l
  ld Ent_spr_h(ix), h
  set_pos ind_4_data, -1, -1
  ld Ent_spr_l(ix), l
  ld Ent_spr_h(ix), h
  ret
;;===========================================================================
;; Initializes the indicators of a level
;; MODIFIES:
;;  CUBE_DATA => Any atribute of the ground button entities can be modified
;; DESTROYS:
;;  HL, AF
;;===========================================================================
indicator_initialize::

  ld  hl, #current_level

  ld  a, #01                  ;;  A=1, compare with level 1
  cp  (hl)                    ;;  |Is A==current_level, if true go to level 1
  jr z, level_1               ;;  \If not, go to next level check

  inc a                       ;;  A++, A=2, compare with level 2
  cp  (hl)
  jr z, level_2

  inc a
  cp  (hl)
  jr z, level_3

  ret

  level_1:
    ld  a, #02
    ld  (NUMBER_OF_INDICATORS), a

    ld  ix, #ind_1_data

    ld     a, #10
    ld     b, #42
    ld    hl, #_sprite_indicator_0
    call  utils_initializeCommon
    ld    Ent_state(ix), #00
    ;;  Which door is connected to, is constant
    ;;  So it will not change on any level, so do not need to initialize it
    ret

  level_2:
  level_3:

  ret

;;===========================================================================
;; Applies a routine to all indicators on the level (or first A of them)
;; DESTROYS: AF, BC, DE, HL
;; INPUT:
;;    A => Number of indicators to apply the routine (or first A of them)
;;    DE => Address of the routine to apply to the indicators
;;===========================================================================
indicator_applyAll::

  push  de                       ;;  Temporally save DE (routine addr) in stack

  ld     hl, #ind_1_x            ;;  HL=ind_data, start of indicator array
  ld_ix_hl                       ;;  IX=HL, saves value of HL in IX

  getEntityOffsetE ind_1, ind_2     ;;  E=grndButton offset, for utils_applyAll
  ld     b, d                    ;;  |B=D, save in B the hgh offset part, BC=DE
  ld     c, e                    ;;  \C=E, save in C the low offset part, BC=DE

  pop   de                       ;;  DE=routineAddr, recover it from  the stack

  ld    a, (NUMBER_OF_INDICATORS) ;  A=04, number of groundButtons to iterate

  jp    utils_applyAll           ;;  Apply groundButton_update to all grndbttns
  

;;===========================================================================
;; Draws an indicator
;; INPUT:
;;    IX => indicator data addr
;; DESTROYS: AF, BC, DE, HL
;;===========================================================================
indicator_draw::
  ld a, Ent_x(ix)             ;; If x == -1
  inc a			;cp #-1
  ret z                       ;; Return

  sub a                       ;;  A=00, draw sprite without transparencies

  ;ld    ix, #ind_data         ;;  IX=ind_data addr, entity start addr

  ld    hl, #_sprite_indicator_1  ;; HL=_sprite_indicator_1, active indc sprite

  ld    Ent_spr_l(ix), l     ;;  EntitySprt=ActiveIndSprite, low part addr
  ld    Ent_spr_h(ix), h     ;;  EntitySprt=ActiveIndSprite, hgh part addr

  jp  utils_drawSprite       ;;  Draws the sprite given in indicator entity

  
;;===========================================================================
;; Erases an indicator, permanent entity, it has own color, not paint backgrnd
;; INPUT:
;;    IX => indicator data addr
;; DESTROYS: AF, BC, DE, HL
;;===========================================================================
indicator_erase::
  ld a, Ent_x(ix)             ;; If x == -1
  inc a                       ;;
  ret z                       ;; Return

  sub a                       ;;  A=00, draw sprite without transparencies

  ;ld    ix, #ind_data         ;;  IX=ind_data addr, entity start addr

  ld    hl, #_sprite_indicator_0  ;; HL=_sprite_indicator_0, NoActive ind sprte

  ld    Ent_spr_l(ix), l     ;;  EntitySprt=NoActiveIndSprite, low part addr
  ld    Ent_spr_h(ix), h     ;;  EntitySprt=NoActiveIndSprite, hgh part addr

  jp  utils_drawSprite       ;;  Draws the sprite given in indicator entity

  
;;===========================================================================
;; Updates an indicator
;; INPUT:
;;    IX => indicator data addr
;; DESTROYS: AF, BC, DE, HL
;;===========================================================================
indicator_update::
  ld a, Ent_x(ix)             ;; If x == -1
  inc a                       ;;
  ret z                       ;; Return

  ld     a, Ent_state(ix)     ;;  A=ind_state value, loads the state
  or a                        ;;  |A == 0? If true, it means desactivated so
  jp    z, paint_desactivated ;;  \paint it own bckgrnd, else paint activated

    ;; Indicator is activated
    call    indicator_draw    ;;  The routine paints it as activated
    ld      a,  #01           ;;  For setting door state as open

    jr end_update

  paint_desactivated:
  call    indicator_erase     ;;  The routine paints it as desactivated
  sub a                       ;;  ;;  For setting door state as closed

  ;;  Set the corresponding door state to A
  end_update:
  push  ix                ;;  Save IX temporally in the stack
  ;ld__d_ixh
  ;ld__e_ixl

  ld    h, Ent_indDoor_h(ix)  ;;  H=doorAddr, high value part loaded
  ld    l, Ent_indDoor_l(ix)  ;;  L=doorrAddr, low value part loaded
  ld_ix_hl                    ;;  IX=HL

  ld c, a                     ;;  C=A, temporally save A value

  sub a                       ;;  A-=A, A is set to 0
  cp  Ent_alwaysActive(ix)    ;;  |If door active by pltRcptr, not change state
  jr nz, update_exit          ;;  \If alwaysActive == 1, active by pltRcptr

    ld  a, c                  ;; A=C, recover A value (the new door state)
    ;call  door_setStateA ;;  |If alwaysActive == 0, set the state that was in A
    ld    Ent_state(ix), a   ;; dr_state=A, set the new door state

  update_exit:
  pop ix
  ;ld__ixh_d
  ;ld__ixl_e
  ret
