.area _DATA
.area _CODE

.include "hero.h.s"
.include "pellet.h.s"                      ;; Energy ball
.include "groundButton.h.s"
.include "cake.h.s"

.include "level_manager.h.s"

.include "cube_gravity.h.s"
.include "gun_portal.h.s"
.include "gun_gravity.h.s"
.include "portals.h.s"
.include "logo.h.s"

;.include "indicator.h.s"
.include "door.h.s"
.include "pellet_receptor.h.s"

.include "cpctelera.h.s"
.include "constants.h.s"
.include "utils.h.s"
.include "keyboard/keyboard.s"

.include "map.h.s"

reset_game:: .db #0

total_timer:: .dw 0x0000
total_deaths:: .dw 0x0000

opt_play: .ascii "  1 Play!\0"
opt_instructions: .ascii "  2 Instructions\0"
opt_credits: .ascii "  3 Credits\0"
opt_exit: .ascii "ESC Exit\0"

.globl _print_stats

instructions:
;;      Line extension:
;;      ====================
.ascii "    Instructions    "
.ascii "                    "
.ascii "                    "
.ascii "                    "
.ascii "Gravity gun:        "
.ascii "                    "
.ascii "  Attract a cube to "
.ascii "                    "
.ascii "  Your position     "
.ascii "                    "
.ascii "                    "
.ascii "Tunnel gun:         "
.ascii "                    "
.ascii "  Shoots a bullet   "
.ascii "                    "
.ascii "  that places a tun-"
.ascii "                    "
.ascii "  nel in the walls  "
.ascii "                    "
.ascii "You can pass from a "
.ascii "                    "
.ascii "tunnel to the other "
.ascii "                    "
.ascii "                    "
.ascii "Press a key to exit "
.ascii "\0"


controls:
;;      Line extension:
;;      ====================
.ascii "      Controls      "
.ascii "                    "
.ascii "                    "
.ascii "                    "
.ascii "Move:       Joystick"
.ascii "                    "
.ascii "Attract cube:  Fire1"
.ascii "                    "
.ascii "Tunnel Gun:    Fire2"
.ascii "                    "
.ascii "Set blue/orange     "
.ascii "tunnel:     Joy down"
.ascii "                    "
.ascii "                    "
.ascii "                    "
.ascii "Press a key to exit "
.ascii "\0"


credits:
;;      Line extension:
;;      ====================
.ascii "      Credits       "
.ascii "                    "
.ascii "                    "
.ascii "                    "
.ascii "Design, implementa- "
.ascii "                    "
.ascii "tion and testing:   "
.ascii "                    "
.ascii "                    "
.ascii "https://github.com/{"
.ascii "                    "
.ascii " emascandela,       "
.ascii "                    "
.ascii " franborjavalero,   "
.ascii "                    "
.ascii " Ivorra }           "
.ascii "                    "
.ascii "                    "
.ascii "2017                "
.ascii "\0"
.globl _sprite_palette

;;============================================================================
;; Initialize the game and systems
;;============================================================================

initialize_game:
  ld     c, #00               ;;  Select video mode 0
  call  cpct_setVideoMode_asm ;;  Set video mode to 0

  ld  hl, #_sprite_palette    ;;  Palette array
  ld  de, #16                 ;;  Palette of 16 colors
  call cpct_setPalette_asm    ;;  Set the new palette of 16 colors

  ld hl, #0x0010
  call cpct_setPALColour_asm

  jp   map_initialize         ;;  Sets the tilset to use


start_game:
  ld a, #1
  call levelManager_setLevel

  call levelManager_startLevel

  ld hl, (level_attempts)
  push hl
  call _print_stats
  pop hl

  ld hl, #0
  ld (level_timer), hl
  ld (level_dtimer), hl

  ret

game_erase:
  call cube_gravity_erase

  ld    ix, #plt_1_data
  call  pellet_erase   ;;  Erases all the pellets

  ld    ix, #plt_2_data
  call  pellet_erase   ;;  Erases all the pellets

  call bullet_erase

  call hero_erase               ;; Erase just before drawing!!!

  jp   gun_portal_erase

game_update:
  call hero_update

  call gun_gravity_update

  call cube_gravity_update

  call gun_portal_update

  ld    ix, #plt_1_data
  call  pellet_update   ;;  Erases all the pellets

  ld    ix, #plt_2_data
  call  pellet_update   ;;  Erases all the pellets

  ld    ix, #grndButton_1_data
  call  groundButton_update

  ld    ix, #grndButton_2_data
  call  groundButton_update

  ld    ix, #grndButton_3_data
  call  groundButton_update

  ;ld    de, #indicator_update
  ;call  indicator_applyAll    ;;  Updates the indictr (grndbuttn active or not)

  ld    ix, #dr_1_data
  call  door_update

  ld    ix, #dr_2_data
  call  door_update

  ld    ix, #dr_3_data
  call  door_update

  ld    ix, #dr_4_data
  call  door_update

  ld    ix, #pltRecept_1_data
  call  pelletReceptor_update

  ld    ix, #pltRecept_2_data
  call  pelletReceptor_update

  ld    ix, #pltRecept_3_data
  call  pelletReceptor_update

  ld    ix, #pltRecept_4_data
  call  pelletReceptor_update

  jp    cake_update

game_move:
  call cube_gravity_move
  call bullet_move

  ld    ix, #plt_1_data
  call  pellet_move   ;;  Erases all the pellets

  ld    ix, #plt_2_data
  call  pellet_move   ;;  Erases all the pellets

  jp    hero_move

game_draw:


  ld    ix, #plt_1_data
  call  pellet_draw   ;;  Erases all the pellets

  ld    ix, #plt_2_data
  call  pellet_draw   ;;  Erases all the pellets

  ;ld de, #pelletReceptor_draw
  ;call pelletReceptor_applyAll   ;;  Draws all receptors with own background color

  call hero_draw
  call cube_gravity_draw

  call gun_portal_draw

  call bullet_draw

  ld    ix, #grndButton_1_data
  call  groundButton_draw

  ld    ix, #grndButton_2_data
  call  groundButton_draw

  ld    ix, #grndButton_3_data
  call  groundButton_draw

  ;;ld    de, #cube_draw
  ;;call  cube_applyAll          ;;  Draws all the cubes

  ;;call obstacle_draw

  call cake_draw
  jp   portals_draw

isr:
  ;; push hl
  ;; push af
  ;;
  ;; ld a, (level_timer)
  ;; inc a
  ;; cp #100
  ;; jr nz, do_not_inc
  ;;
  ;; ld hl, (level_dtimer)
  ;; inc hl
  ;; ld (level_dtimer), hl
  ;;
  ;; ld a, #0
  ;; ld (level_timer), a
  ;;
  ;; pop af
  ;; pop hl
  ;; ret
  ;;
  ;; do_not_inc:
  ;; ld (level_timer), a
  ;; pop af
  ;; pop hl
  ret

main_init:
  call initialize_game

  ;ld    de, #indicator_erase
  ;call  indicator_applyAll    ;;  Draws all indicators with own bckgrnd color

  ld    de, #door_erase
  call  door_applyAll    ;;  Draws all indicators with own bckgrnd color

  ld    de, #pelletReceptor_erase
  jp    pelletReceptor_applyAll   ;;  Draws all receptors with own background color

print_logo_animation:
  logo_animation_loop:
    call logo_erase
    call logo_draw
    call logo_move
    call cpct_waitVSYNC_asm
    halt
    halt
    halt
    call cpct_waitVSYNC_asm
    halt
    halt
    halt
    call cpct_waitVSYNC_asm
    ld a, Ent_y(ix)
    cp #30
    jr nz, logo_animation_loop
  ret

print_menu_options:
  ld de, #0xC000
  ld bc, #0x6408
  call cpct_getScreenPtr_asm
  ex de, hl
  ld hl, #opt_play
  ld bc, #0x070B
  call cpct_drawStringM0_asm

  ld de, #0xC000
  ld bc, #0x7408
  call cpct_getScreenPtr_asm
  ex de, hl
  ld hl, #opt_instructions
  ld bc, #0x070B
  call cpct_drawStringM0_asm

  ld de, #0xC000
  ld bc, #0x8408
  call cpct_getScreenPtr_asm
  ex de, hl
  ld hl, #opt_credits
  ld bc, #0x070B
  call cpct_drawStringM0_asm

  ld de, #0xC000
  ld bc, #0x9408
  call cpct_getScreenPtr_asm
  ex de, hl
  ld hl, #opt_exit
  ld bc, #0x070B
  call cpct_drawStringM0_asm
  ret

checkUserInput:
  call cpct_scanKeyboard_asm

  ld hl, #Key_1
  call cpct_isKeyPressed_asm
  or a
  jr z, no_key_1
    ld a, #1
    ret
  no_key_1:

  ld hl, #Key_2
  call cpct_isKeyPressed_asm
  or a
  jr z, no_key_2
    ld hl, #instructions
    call utils_print
    ld hl, #controls
    call utils_print
    ld a, #2
    ret
  no_key_2:

  ld hl, #Key_3
  call cpct_isKeyPressed_asm
  or a
  jr z, no_key_3
    ld hl, #credits
    call utils_print
    ld a, #2
    ret
  no_key_3:

  ld hl, #Key_Esc
  call cpct_isKeyPressed_asm
  or a
  ret z
    ld a, #-1
    ret
 
main_menu:
  call utils_eraseScreen
  call print_logo_animation
  draw_menu:
    call logo_draw
    call print_menu_options
  menu_loop:
    call checkUserInput
    cp #2
    jr z, draw_menu
    or a
    jr z, menu_loop
  ret

;;========================================
;; Main program entry
;;========================================
_main::
call cpct_disableFirmware_asm
call  main_init

ld hl, #isr
call cpct_setInterruptHandler_asm


menu:

ld hl, #0x0000
ld (total_deaths), hl

sub a
ld (reset_game), a
ld ix, #logo_data
ld Ent_y(ix), a

call main_menu
inc a			;cp #-1
jr z, end_game

call start_game


gameLoop:

  call game_update
  call cpct_waitVSYNC_asm

  call levelManager_update

  call game_erase
  call game_move
  call game_draw

  ld a, (reset_game)
  dec a			;cp #1
  jr z, menu

  ;; ld hl, (hero_deaths)
  ;; push hl
  ;; call _print_stats

  ld hl, #level_timer
  inc (hl)

  jr gameLoop                ;; infinite loop

end_game:
