.area _CODE

;;=============================================================================
;; Includes
;;=============================================================================

.include "cpctelera.h.s"
.include "utils.h.s"
.include "constants.h.s"

.include "portals.h.s"
.include "map.h.s"

.globl current_level

;;=============================================================================
;; Private data
;;=============================================================================
;plt_data:
;  plt_x:        .db #60
;  plt_y:        .db #86
;  plt_w:        .db #03
;  plt_h:        .db #06
;  plt_spr:      .dw _sprite_pellet
;  plt_state:    .db #01       ;;  Pellet state, active or not (1 or 0)
;  plt_sense:    .db #-1       ;;  Sense of movement (-1 to left, 1 to right)

  definePellet plt_1, -1, -1, 02, 08, #_sprite_pellet2, 0, 0, 0
  definePellet plt_2, -1, -1, 02, 08, #_sprite_pellet2, 0, 0, 0
  definePellet plt_3, -1, -1, 02, 08, #_sprite_pellet2, 0, 0, 0
  definePellet plt_4, -1, -1, 02, 08, #_sprite_pellet2, 0, 0, 0

NUMBER_OF_PELLETS:: .db #02

;;=============================================================================
;; Public functions
;;=============================================================================

;;===========================================================================
;; Uninitializer the pellets of a level
;; MODIFIES:
;;  PELLET_DATA => Any atribute of the pellet entities can be modified
;; DESTROYS:
;;  HL, AF
;;===========================================================================
pellet_uninitialize::
  set_pos plt_1_data, -1, -1
  set_vel_ix 0, 0
  ld Ent_pltState(ix), #00
  set_pos plt_2_data, -1, -1
  set_vel_ix 0, 0
  ld Ent_pltState(ix), #00
  set_pos plt_3_data, -1, -1
  set_vel_ix 0, 0
  ld Ent_pltState(ix), #00
  set_pos plt_4_data, -1, -1
  set_vel_ix 0, 0
  ld Ent_pltState(ix), #00

  ret

;;===========================================================================
;; Applies a routine to all pellets on the level (or first A of them)
;; DESTROYS: AF, BC, DE, HL
;; INPUT:
;;    A => Number of pellets to apply the routine (or first A of them)
;;    DE => Address of the routine to apply to the pellets
;;===========================================================================
pellet_applyAll::

  push  de                       ;;  Temporally save DE (routine addr) in stack

  ld hl, #plt_1_x                ;;  HL=dr_data, start of pellet array
  ld_ix_hl                       ;;  IX=HL, saves value of HL in IX

  getEntityOffsetE plt_1, plt_2  ;;  E=pellet offset, for utils_applyAll
  ld     b, d                    ;;  |B=D, save in B the hgh offset part, BC=DE
  ld     c, e                    ;;  \C=E, save in C the low offset part, BC=DE

  pop   de                       ;;  DE=routineAddr, recover it from  the stack

  ld    a, (NUMBER_OF_PELLETS)   ;;  A=04, number of pellets to iterate

  jp    utils_applyAll           ;;  Apply groundButton_update to all pellets
  
;;===========================================================================
;; Draws a pellet if its state is activate, else do nothing
;; INPUT:
;;    IX => pellet data addr
;; DESTROYS: AF, BC, DE, HL
;;===========================================================================
pellet_draw::
  ld a, Ent_x(ix)           ;; if x == -1
  inc a                     ;;
  ret z                     ;; return

  ld     a, Ent_pltState(ix)   ;;  A=plt_state, put in A the state value
  or	    a                  ;;  If A=plt_state=0, not updte, else if A=1, updte
  ret  z                    ;;  If A==0, return, else update

  ld     a, #01             ;;  A=01, draw sprite with transparencies

  ;ld   ix, #plt_data        ;;  IX=plt_data addr, entity start addr
  jp    utils_drawSprite     ;;  Draws the sprite given in pellet entity

;;===========================================================================
;; Erases a pellet if its state is activate, else do nothing
;; INPUT:
;;    IX => pellet data addr
;; DESTROYS: AF, BC, DE, HL
;;===========================================================================
pellet_erase::
  ld a, Ent_x(ix)           ;; if x == -1
  inc a                     ;;
  ret z                     ;; return

  ld     a, Ent_pltState(ix)   ;;  A=plt_state, put in A the state value
  or   a                    ;;  If A=plt_state=0, not updte, else if A=1, updte
  ret  z                    ;;  If A==0, return, else update

  ld c, Ent_x(ix)
  ld b, Ent_y(ix)
  ld de, #0xC000
  call cpct_getScreenPtr_asm
  ex de, hl
  ld a, #bg_color
  ld c, Ent_w(ix)
  ld b, Ent_h(ix)
  jp cpct_drawSolidBox_asm
  ;ld     a, #01             ;;  A=01, draw sprite with transparencies
  ;call utils_drawSprite     ;;  Draws the sprite given in pellet entity

;;=============================================================================
;; Modifies pellet position
;; INPUT:
;;    IX => pellet data addr
;; DESTROYS: A, C, HL
;; MODIFIES:
;;  plt_x => Increases by 1 (move right) if plt state is 1, else by -1, to left
;;=============================================================================
;; pellet_update::
;;   ld a, Ent_x(ix)           ;; if x == -1
;;   cp #-1                    ;;
;;   ret z                     ;; return
;;
;;   ;; Move pellet to the left if state is -1 and to the right if state is 1
;;
;;   ld     c, Ent_x(ix)        ;;  C=plt_x value, pellet x coords
;;
;;   ;ld    hl, #plt_x          ;;  HL=plt_x addr, for obtaining in C plt_x value
;;   ;ld     c, (hl)            ;;  C=plt_x value, pellet x coords
;;
;;   ld     a, Ent_sense(ix)       ;;  A=plt_sense, put in A the sense value
;;   ;;call  pellet_getSenseA  ;;  A=plt_state, for adding it to C,
;;   add    c                ;;  A+=plt_sense, if state=1 move rght, if -1, left
;;   ;;   ld (hl), a
;;
;;   jr     z, invert_movement ;;  It reaches left wall, so invert movement sense
;;   cp    #80-1               ;;  It reaches right wall, so invert movement sense
;;   jr    z, invert_movement  ;;
;;
;;     ;; No need to invert movement
;;     ld    Ent_x(ix), a      ;;  Ent_x=A, update X coordinate, X+1 or X-1
;;
;;     ret
;;
;;   invert_movement:
;;   call  pellet_invertState
;;
;;   ret

pellet_update::
  ld a, Ent_x(ix)           ;; if x == -1
  inc a                     ;;
  ret z                     ;; return

  ld     a, Ent_pltState(ix)   ;;  A=plt_state, put in A the state value
  or   a                    ;;  If A=plt_state=0, not updte, else if A=1, updte
  ret  z                    ;;  If A==0, return, else update

  call portals_crossIfInside
  cp #-1
  call nz, setOrientation

  ld a, Ent_vx(ix)
  or a
  jr nz, horizontal_direction
  ld a, Ent_vy(ix)
  or a
  jr nz, vertical_direction
  ret

  horizontal_direction:
  dec a
  jr nz, left_sense

  right_sense:
    call map_checkRightBounds
    dec a
    ret nz

    ld Ent_vx(ix), #-1
    ret

  left_sense:
    call map_checkLeftBounds
    dec a
    ret nz

    ld Ent_vx(ix), #1
    ret

  ret

  vertical_direction:
  cp #4
  jr z, down_sense
 
  up_sense:
    call map_checkUpperBounds
    dec a
    ret nz

    ld Ent_vy(ix), #4
    ret

  down_sense:
    call map_checkLowerBounds
    dec a
    ret nz

    ld Ent_vy(ix), #-4
    ret

  ret

pellet_move::
  ld     a, Ent_pltState(ix)   ;;  A=plt_state, put in A the state value
  or     a                  ;;  If A=plt_state=0, not updte, else if A=1, updte
  ret  z                    ;;  If A==0, return, else update

  jp utils_moveEntity


;;=============================================================================
;; Check if another entity collides with pellet, entity is passed in HL
;; DESTROYS: A, HL, DE
;; INPUTS:
;;  HL => Pointer to begin of entity, check if entity collides with a pellet
;;  DE => Actual cube address
;; RETURNS:
;;  A => Collision value, A=0x00 if collision, A=0xFF if there is no collision
;;=============================================================================
pellet_isCollisionAll::
  ld de, #plt_1_data
  call  utils_checkCollision  ;;  Check if collide entity and pellet_1
  or a
  ret nz

  ld de, #plt_2_data
  call  utils_checkCollision  ;;  Check if collide entity and pellet_2
  or a
  ret nz

  ;ld de, #plt_3_data
  ;call  utils_checkCollision  ;;  Check if collide entity and pellet_3
  ;cp  #00
  ;ret nz

  ;ld de, #plt_4_data
  ;call  utils_checkCollision  ;;  Check if collide entity and pellet_4
  ;cp  #00
  ;ret nz

  sub a       ;; NO COLLISION
  ret


setOrientation:
  cp #Down
  jr z, set_to_down
  cp #Up
  jr z, set_to_up
  cp #Left
  jr z, set_to_left
  cp #Right
  jr z, set_to_right
  ret

  set_to_down:
    ld Ent_vx(ix), #0
    ld Ent_vy(ix), #-4
    ret

  set_to_up:
    ld Ent_vx(ix), #0
    ld Ent_vy(ix), #4
    ret

  set_to_left:
    ld Ent_vx(ix), #1
    ld Ent_vy(ix), #0
    ret

  set_to_right:
    ld Ent_vx(ix), #-1
    ld Ent_vy(ix), #0
    ret
