.area _CODE

;;=============================================================================
;; Includes
;;=============================================================================

.include "cpctelera.h.s"
.include "utils.h.s"
.include "constants.h.s"

.include "pellet.h.s"
.include "door.h.s"

.globl current_level

;;.include "hero.h.s"

;;============================================================================
;; Private data
;;============================================================================
;pltRecept_data:
;  pltRecept_x:        .db #30
;  pltRecept_y:        .db #52
;  pltRecept_w:        .db #03
;  pltRecept_h:        .db #08
;  pltRecept_sprite:   .dw #_sprite_pelletReceptor_0
;  pltRecept_state:    .db #00 ;;  Activate or not (0 not active, 1 active)

;;============================================================================
;; Public functions
;;============================================================================

defineIndicator pltRecept_1, -1, -1, 03, 12, #_sprite_pelletReceptor_0, 0, 1
defineIndicator pltRecept_2, -1, -1, 03, 12, #_sprite_pelletReceptor_0, 0, 2
defineIndicator pltRecept_3, -1, -1, 03, 12, #_sprite_pelletReceptor_0, 0, 3
defineIndicator pltRecept_4, -1, -1, 03, 12, #_sprite_pelletReceptor_0, 0, 4

NUMBER_OF_PLTRECEPTORS:: .db #02

;;===========================================================================
;; Uninitializes the pelet receptors of a level
;; MODIFIES:
;;  PLTRECEPTOR_DATA => Any atribute of the pellet entities can be modified
;; DESTROYS:
;;  HL, AF
;;===========================================================================
pelletReceptor_uninitialize::
  ld hl, #_sprite_pelletReceptor_0
  set_pos pltRecept_1_data, -1, -1
  ld Ent_spr_l(ix), l
  ld Ent_spr_h(ix), h
  ld Ent_state(ix), #00
  set_pos pltRecept_2_data, -1, -1
  ld Ent_spr_l(ix), l
  ld Ent_spr_h(ix), h
  ld Ent_state(ix), #00
  set_pos pltRecept_3_data, -1, -1
  ld Ent_spr_l(ix), l
  ld Ent_spr_h(ix), h
  ld Ent_state(ix), #00
  set_pos pltRecept_4_data, -1, -1
  ld Ent_spr_l(ix), l
  ld Ent_spr_h(ix), h
  ld Ent_state(ix), #00



  ret
;;===========================================================================
;; Initializes the pelet receptors of a level
;; MODIFIES:
;;  PLTRECEPTOR_DATA => Any atribute of the pellet entities can be modified
;; DESTROYS:
;;  HL, AF
;;===========================================================================
pelletReceptor_initialize::

  ld  hl, #current_level

  ld  a, #01                  ;;  A=1, compare with level 1
  cp  (hl)                    ;;  |Is A==current_level, if true go to level 1
  jr z, level_1               ;;  \If not, go to next level check

  inc a                       ;;  A++, A=2, compare with level 2
  cp  (hl)
  jr z, level_2

  inc a
  cp  (hl)
  jr z, level_3

  ret

  level_1:
    ld  a, #02                ;;  A=01, number of pellets in the first level
    ld  (NUMBER_OF_PLTRECEPTORS), a  ;;  One pellet in the first level

    ld  ix, #pltRecept_1_data      ;;  IX=cube_1_data, first cube

    ld     a, #20
    ld     b, #96

    ld    hl, #_sprite_pelletReceptor_0
    call  utils_initializeCommon
    ld    Ent_state(ix), #00

    ret

  level_2:
  level_3:

  ret

;;===========================================================================
;; Applies a routine to all pellet receptors on the level (or first A of them)
;; DESTROYS: AF, BC, DE, HL
;; INPUT:
;;    DE => Address of the routine to apply to the pellet receptors
;;===========================================================================
pelletReceptor_applyAll::
  push  de                       ;;  Temporally save DE (routine addr) in stack

  ld hl, #pltRecept_1_x          ;;  HL=cube_data, start of door array
  ld_ix_hl                       ;;  IX=HL, saves value of HL in IX

  getEntityOffsetE pltRecept_1, pltRecept_2 ;  E=cube offset, for utils_applyAll
  ld     b, d                    ;;  |B=D, save in B the hgh offset part, BC=DE
  ld     c, e                    ;;  \C=E, save in C the low offset part, BC=DE

  pop   de                       ;;  DE=routineAddr, recover it from  the stack

  ld     a, (NUMBER_OF_PLTRECEPTORS)

  jp    utils_applyAll           ;;  Apply groundButton_update to all doors

;;===========================================================================
;; Draws a pellet receptor
;; INPUT:
;;    IX => pellet receptor data addr
;; DESTROYS: AF, BC, DE, HL
;;===========================================================================
pelletReceptor_draw::
  ld a, Ent_x(ix)            ;; if x == -1
  inc a                      ;;
  ret z                      ;; return

  sub a                      ;;  A=00, draw sprite without transparencies
  ;ld    ix, #pltRecept_data  ;;  IX=ind_data addr, entity start addr

  ld    hl, #_sprite_pelletReceptor_1  ;; HL=_sprite_indicator_1, active indc sprite

  ld    Ent_spr_l(ix), l     ;;  EntitySprt=ActiveReceptSprite, low part addr
  ld    Ent_spr_h(ix), h     ;;  EntitySprt=ActiveReceptSprite, hgh part addr

  jp   utils_drawSprite      ;;  Draws the sprite given in receptor entity

;;===========================================================================
;; Erases a pltReceptor, permanent entity, it has own color, not paint backgrnd
;; INPUT:
;;    IX => pellet receptor data addr
;; DESTROYS: AF, BC, DE, HL
;;===========================================================================
pelletReceptor_erase::
  ld a, Ent_x(ix)            ;; if x == -1
  inc a                      ;;
  ret z                      ;; return

  sub a                      ;;  A=00, draw sprite without transparencies

  ;ld    ix, #pltRecept_data  ;;  IX=ind_data addr, entity start addr

  ld    hl, #_sprite_pelletReceptor_0  ;; HL=_sprite_indicator_1, active indc sprite

  ld    Ent_spr_l(ix), l     ;;  EntitySprt=ActiveReceptSprite, low part addr
  ld    Ent_spr_h(ix), h     ;;  EntitySprt=ActiveReceptSprite, hgh part addr

  jp   utils_drawSprite      ;;  Draws the sprite given in receptor entity

;;===========================================================================
;; Updates a pellet receptor, check if pellet collides it, if true, activate it
;; DESTROYS: A, HL, DE
;; INPUT:
;;    IX => pellet receptor data addr
;; MODIFIES:
;;    pltRecept_state => If plt collides receptor activate pellet receptor
;;    plt_sense => If plt collides receptor, desactivate pellet
;;===========================================================================
pelletReceptor_update::
  ld a, Ent_x(ix)            ;; if x == -1
  inc a                      ;;
  ret z                      ;; return

  ;call  pellet_getPtrHL        ;;  Set HL to init of pllet dimensions structre
  ;call  isCollisionPltReceptor ;; See if pellet collide cube, A=0 FALSE, 1 TRUE

  ;call  pelletReceptor_getStateA ;;  Set pellet_state in A
  ld     a, Ent_state(ix)       ;;  A=pltRecept_state, put in A state value
  dec a                       ;;  If A=rcpt_state=1, only draw, already active
  jr  z, pelletReceptor_draw    ;;  If A==0, return, else draw

  sub a                        ;;  A=00, ensure default is no collision
  ld_hl_ix                     ;;  HL=IX, for using as argument of next routine

  call  pellet_isCollisionAll ;;  If collide A=0xFF and DE=pelletColliding
  ;call  pellet_getPtrHL        ;;  Set HL to init of pllet dimensions structre
  ;call  isCollisionPltReceptor ; See if pellet collide cube, A=0 FALSE, 1 TRUE

  or a	                     ;;  |If collisn, activate receptor (state to 1)
  ;jr     nz, collision        ;;  \If no collisn, desaactive it (state to 0)

    ;; There is no collision, keep recetor drawn as not active
    jr z, pelletReceptor_erase ;  Draw the receptor as its background colors
    ;; NO NEED TO DRAW DOOR AS CLOSED, IT IS IN THIS WAY BY DEFAULT


  ;; There is collision, desactivate plt, activate rcptor, open correct door
  collision:
  push ix                     ;; Temporally save IX in the stack

  ld__ixh_d                   ;; IX = pellet_addr the one that collides
  ld__ixl_e                   ;; IX = pellet_addr the one that collides

  call pellet_erase           ;; Delete pellet when it collides the receptor

  sub a                       ;;  A=00, desactivte plt by setting pltState to 0
  ld    Ent_pltState(ix), a   ;; Plt_state=A, set pellet to not active

  ld    Ent_x(ix), a          ;; Move pellet to a position where hero never is
  ld    Ent_y(ix), a          ;; Move pellet to a position where hero never is
  ;ld__iyh_d
  ;ld__iyl_e
  pop  ix

  ;call  pellet_setStateAIY   ;;  Change pellet state value to A value


  call  pelletReceptor_draw   ;;  Draw the receptor as its activate colors

  ld  a, #01
  ld    Ent_state(ix), a      ;; pltRecept_state=A, set new receptr state
  ;call pelletReceptor_setStateA

  ;ld__d_ixh
  ;ld__e_ixl
  push  ix                    ;;  Save IX temporally in the stack

  ld    h, Ent_pltRecpDoor_h(ix)  ;;  H=doorAddr, high value part loaded
  ld    l, Ent_pltRecpDoor_l(ix)  ;;  L=doorrAddr, low value part loaded
  ld_ix_hl                    ;;  IX=HL

  ;ld  a, #01

  ld    Ent_state(ix), a   ;; dr_state=A, set the new door state
  ;call  door_setStateA      ;  Sets the state that was in A

  ld    Ent_alwaysActive(ix), a    ;; dr_alwaysActive=01, door open by rcptr
  ;call  door_setAlwaysActive

  ;ld__ixh_d
  ;ld__ixl_e
  pop  ix                 ;;  Recover IX from the stack
  ret

  
;;=============================================================================
;; Private functions
;;=============================================================================
