.area _DATA

;;=============================================================================
;; Includes
;;=============================================================================
.include "cpctelera.h.s"
.include "constants.h.s"
;;=============================================================================
;; Draw a box of given dimensions
;; INPUTS:
;;     A  => Colour pattern
;;    BC => (height, width), (pixels, bytes), hgt bye == px, wdt byte == 4 px
;;    DE => CoordX tag addr
;;    HL => CoordY tag addr
;; DESTROYS: AF, BC, DE, HL
;;=============================================================================
utils_drawBox::
  push  af                            ;;  Put A (colour) in stack, op use 2byts
  push  bc                            ;;  Put BC (box size) in stack

  ;; Calculate screen position
  ld     a, (de)                       ;;  | A=CoordX in DE saved tag addr
  ld     c, a                          ;;  \ C=A, cordX to convrt to scren addr

  ld     a, (hl)                       ;;  | A=CoordY in HL saved tag addr
  ld     b, a                          ;;  \ B=A, cordY to convrt to scren addr

  ld    de, #0xC000                    ;;  Video memry ptr, scren strts at C000

  call  cpct_getScreenPtr_asm          ;;  Get ptr to screen, RETURNED in HL

  ;; Draw a Box, DE needs addr to where to start to paint
  ex    de, hl                        ;;  Locatn was in HL, SwapVals, now in DE
  pop   bc                            ;;  Recover in A, the saved colour pattrn
  pop   af                            ;;  Recover in BC, the saved box dims
  jp    cpct_drawSolidBox_asm         ;;  Draw the box


;;=============================================================================
;; Check collision between two entities (indicated by its memory addresses)
;; NOT MODIFIES THE MEMORY (ONLY READ THE ENTITIES' VALUES)
;; INPUTS:
;;    HL => Pointer to the beggining of first entity (2bytes)
;;    DE => Pointer to the beggining of second entity (2bytes)
;; DESTROYS: A, (HL, DE ??)
;;============================================================================
utils_checkCollision::
  call utils_IsIntersectionInOneAxis  ;;  Check if there is X COLLISION
  or a                                ;;  |Is A == 0? True, means no collision
  ret z			                   ;;  \IF A == 1, would mean collision
  

    ;; There is collision in X Axis, as A is 1
    inc   hl                          ;;  |HL++, move to second value, Y coord
    ;;inc   hl   NO NEED, Y IS SECOND NOT THIRD ;;  \HL++, move to third (Y coordinate)

    inc   de                          ;;  |DE++, move to second value, Y coord
    ;;inc   de   NO NEED, Y IS SECOND NOT THIRD ;;  \DE++, move to third (Y coordinate)

    call utils_IsIntersectionInOneAxis  ;;  Check if there is Y COLLISION
    dec hl
    dec de
    neg                                 ;;  A remains 0 if no collision
    						           ;;  A set to -1 if collision
    ret

;;=============================================================================
;; Draw an entity given its address inside IX
;; INPUTS:
;;     A => If A==0, draw sprite without transparencies, if A==1 draw with them
;;    IX => Address that points to the beginning of the entity
;; DESTROYS: AF, BC, DE, HL
;;=============================================================================
utils_drawSprite::
  push  af                            ;;  Save A in stack, transparent or not

  ld    de, #0xC000                   ;;  Video memry ptr, scren strts at C000

  ld     c, Ent_x(ix)                 ;;  |C=ent_x, cordX, convrt to scren addr
  ld     b, Ent_y(ix)                 ;;  \Y=ent_y, cordY, convrt to scren addr

  call cpct_getScreenPtr_asm          ;;  Get ptr to screen, returned in HL

  ex    de, hl                        ;;  ScreenPtr in HL, SwapVals, now in DE

  ld     h, Ent_spr_h(ix)             ;;  H=spriteAddr, high value part loaded
  ld     l, Ent_spr_l(ix)             ;;  L=spriteAddr, low value part loaded

  ld     c, Ent_w(ix)                 ;;  C=wdthValue, Value from Addr=EntW+IX
  ld     b, Ent_h(ix)                 ;;  B=hghtValue, Value from Addr=EntH+IX

  pop   af                            ;;  Recover A from stack, trnsprnt or not
  dec    a                            ;;  |A--, if A==1, draw transparent
                                      ;;  \Else if A==0, draw not transparent
  jp     z, cpct_drawSpriteMasked_asm      ;;  If zero, A was 1, so draw transparent

    ;; A was not 1, so draw with no transparecy, as normal (not masked) sprite
    jp   cpct_drawSprite_asm          ;;  Draw an sprite without transparencies


;;=============================================================================
;; Moves an entity
;; INPUTS:
;;    IX => Pointer to the entity
;;============================================================================
utils_moveEntity::
    ld a, Ent_x(ix)             ;; |
    add Ent_vx(ix)              ;; |
    ld Ent_x(ix), a             ;; \ Adds vx to x

    ld a, Ent_y(ix)             ;; |
    add Ent_vy(ix)              ;; |
    ld Ent_y(ix), a             ;; \ Adds vy to y

    ret

utils_eraseScreen::
  ld c, #64
  ld b, #200
  ld de, #0xC000
  ld a, #0xFC
  call cpct_drawSolidBox_asm
  ld bc, #0x0040
  ld de, #0xC000
  call cpct_getScreenPtr_asm
  ex de, hl
  ld c, #16
  ld b, #200
  ld a, #0xFC
  jp  cpct_drawSolidBox_asm


;;=============================================================================
;; Prints a text in screen. Ends when a key is pressed
;;=============================================================================
utils_print::
  push hl
  call utils_eraseScreen
  pre_print_loop:
    call cpct_scanKeyboard_asm
    call cpct_isAnyKeyPressed_asm
    or a
    jr nz, pre_print_loop

  pop hl
  ld de, #0xC000
  ld bc, #0x070B
  call cpct_drawStringM0_asm

  print_loop:
    call cpct_scanKeyboard_asm
    call cpct_isAnyKeyPressed_asm
    or a
    jr z, print_loop
  jr utils_eraseScreen

;;=============================================================================
;; Applies a routine over an array of entities
;; INPUTS:
;;    IX => Pointer to the beggining of the entity array
;;    DE => Pointer to the routine
;;    BC => Offset, distance between two consecutive entities
;;     A => Counter, number of entities to iterate
;; DESTROYS: AF, BC, DE, HL
;;============================================================================
utils_applyAll::

  or a                              ;;  Check if all entities are iterated
  ret z					       ;;  If A==0, all are iterated, else contnue

    ;; Count has not finished, so we still have to draw more hero entities
    push  af                        ;; Save counter value in the stack
    push  bc                        ;; Save offset value in the stack
    push  de                        ;; Save routine address in the stack

    ld  b, #00                      ;;  B=00, for next addition with PC
    ld  c, #1+1+1+1+1               ;;  C=5, bytes to move PC after jp (HL)

    call nextInstruction            ;;  For obtaining next PC in HL
    nextInstruction:                ;;  From one instruction go to the next
    pop hl                          ;;  HL=PC, points to start of this instrctn

    add hl, bc                      ;;  HL+=BC, HL=PC+5, HL points aftr jp (HL)
    ex de, hl                       ;;  PC was in HL, SwapVals, now in DE
    push de                         ;;  Put PC in stack, RET will pop it later
    jp (hl)                         ;;  PC=HL value, so we jump to the routine

    pop   de                        ;; Recover routine address from the stack

    pop   bc                        ;; Recover offset value from the stack
    ld     b, #00                   ;; B=00, clear B value in addition IX+BC
    add   ix, bc                    ;; IX+=BC, EntitAddr+Offset, move to next

    pop    af                       ;; Recover counter value from the stack
    dec    a                        ;; A--, decrease by one the loop counter

    jr    utils_applyAll+1


;;=============================================================================
;; Moves an entity
;; INPUTS:
;;    IX => Pointer to the entity
;;     A => X coordinate
;;     B => Y coordinate
;;     H => Sprite address, high part
;;     L => Sprite address, low part
;;============================================================================
utils_initializeCommon::
  ld  Ent_x(ix), a
  ld  Ent_y(ix), b

  ld  Ent_spr_h(ix), h
  ld  Ent_spr_l(ix), l

  ret

;;=============================================================================
;; Private functions
;;=============================================================================


;;=============================================================================
;; Check if there is collision in a given axis
;;    Uses two times the formula: (originOne + lengthOne) <= originTwo,
;;    In the form: ((originOne + lengthOne) - originTwo)) <= 0   (<, no touch)
;;    Second time is: ((originTwo + lengthTwo) - originOne) <= 0 (=, touch)
;; INPUTS:                                                       (>, collide)
;;    HL => Pointer to the first axis origin's value, HL+1 is length of entity
;;    DE => Pointer to the second axis origin's value, DE+1 is length of entity
;;
;; RETURNS:
;;    A => whether if there is intersection or not (1 if true, O if false)
;;    HL and DE => They are restored to the input values
;;============================================================================
utils_IsIntersectionInOneAxis:
  ;; Compute first: ((originOne + lengthOne) - originTwo)) <= 0
  ld     a, (hl)                  ;;  A=originOne value
  inc   hl                        ;;  HL++, go to second, third is len addr
  inc   hl   ;;   NEW LINE        ;;  HL++, go to len addr, +2 dstnce of orign

  add   (hl)                      ;;  A+=(HL), originOne+lengthOne
  ex    de, hl                    ;;  DE swap HL, len1Addr in DE, orig2Addr HL
  sub   (hl)                      ;;  A-=HL == origOne + lenOne - origTwo

  ;; Restore pointers before jumping
  ;; Now HL=origTwoAddr, HL+2=lengthTwoAddr, DE-2=origOneAddr, DE=lengthOneAddr

  ex    de, hl                    ;;  DE swap HL, orig2Addr in DE, len1Addr HL
  dec   hl                        ;;  HL--, HL is len1Addr addr, now orig!1Addr
  dec   hl   ;;   NEW LINE        ;;  HL--, HL is orig!1Addr, now orig1Addr

  jr     z, no_intersection       ;;  If substrctn is 0, there is not collision
  jp     m, no_intersection       ;;  Same if substraction is negative

    ;; There is collision, substraction is greater than 0 (not 0 nor negative)

    ;; Prepare pointers for second operation
    ex    de, hl                  ;; DE swap HL, orig1Addr in DE, orig2Addr HL

    ;; Compute second: ((originTwo + lengthTwo) - originOne) <= 0
    ld     a, (hl)                ;;  A=originTwo value
    inc   hl                      ;;  HL++, go to second, third is len addr
    inc   hl   ;;   NEW LINE      ;;  HL++, go to len addr, +2 dstnce of orign

    add   (hl)                    ;;  A+=(HL), originTwo+lengthTwo
    ex    de, hl                  ;;  DE swap HL, len2Addr in DE, orig1Addr HL
    sub   (hl)                    ;;  A-=HL == origTwo + lenTwo - origOne

    ;; Restore pointers before jumping
    ;; Now HL=origOneAddr, DE=lengthTwoAddr
    ;;  ex    de, hl    NO, AS ALREADY EXCHANGED TWO OPERATIONS ABOVE.
    dec   de                      ;; DE--, DE is lengt2Addr, now orig!2Addr
    dec   de   ;;   NEW LINE      ;; DE--, DE is orig!2Addr, now orig2Addr

    jr     z, no_intersection     ;;  If substrctn is 0, there is not collision	
    jp     m, no_intersection     ;;  Same if substraction is negative

    ld     a, #01                 ;;  A=0, if there is no collision, set A to 0
    ret

  no_intersection:
  sub a                           ;;  A=0, if there is no collision, set A to 0

  ret

;;============================================================================
;;============================================================================
;;============================================================================
;;============================================================================
;;============================================================================
;;============================================================================

;;========================================
;; Public methods
;;========================================
;;========================================
;; COLLISION CHECK
;; Indicate when produce collision between two objects passed by parameters
;; The first conditions will be check the axis which have higher probability
;; to crash with the objecta passed by parameters
;; INPUTS:
;;  hl points hero
;;  de points obstacle
;; DESTROYS:
;;
;;========================================

collision_check::
  ;; AXIS X: de is behind hl
  ;; if (obs_x + obs_w <= hero_x) no_collision
  call collision_formula            ;; de = obstacle_x, hl = hero_x
  jr z, no_collision
  jp m, no_collision
  ;; AXIS X: de is ahead hl
  ;; if (hero_x + hero_w <= obs_x) no_collision
  dec de                            ;; de points obstacle_y
  dec de                            ;; de points obstacle_x
  ex de, hl                         ;; swap
  call collision_formula            ;; de = hero_x, hl = obstacle_x
  jr z, no_collision
  jp m, no_collision
  ;; AXIS Y: de is above hl
  ;; if (hero_y + hero_h <= obs_y) no_collision
  dec de                            ;; de points obstacle_y
  inc hl                            ;; hl points hero_y
  ex de, hl                         ;; swap
  call collision_formula            ;; de = obstacle_y, hl = hero_y
  jr z, no_collision
  jp m, no_collision
  ;; AXIS Y: de is above hl
  ;; if (hero_y + hero_h <= obs_y) no_collision
  dec de                            ;; de points obstacle_w
  dec de                            ;; de points obstacle_y
  ex de, hl                         ;; swap
  call collision_formula            ;; de = obstacle_x, hl = hero_x
  jr z, no_collision
  jp m, no_collision
  ld a, #0xFF
  ret
  no_collision:
  sub a
  ret


;;========================================
;; Private methods
;;========================================

;;========================================
;; Add operation distance two objects
;; INPUTS:
;;  de (obstacle)
;;  hl (hero)
;; OUTPUT: flag z ¿?
;;========================================
collision_formula:
  ld a, (de)          ;; obstacle_x
  ld c, a             ;; C = obstacle_x
  inc de              ;; de points obstacle_y
  inc de              ;; de points obstacle_w
  ld a, (de)          ;; A = obstacle_w
  add c               ;; obs_w + obs_x
  sub (hl)            ;; A - value direction hl (whose value is hero_x)
                      ;; obviously modify the flags
                      ;;ld c, (hl)      ;; permit load in all registers directly (hl)
                      ;;sub c
  ret
